@@ -1,8 +1,8 @@
 #include "config.h"
 #include <stdio.h>
 #include <stdlib.h>
-#include <stdbool.h>
 #include <unistd.h>
+#include <time.h>
 #include <dirent.h>
 #include <pthread.h>
 #include <sys/types.h>
@@ -10,11 +10,12 @@
 #include "client.h"
 #include "log.h"
 
+#define ip_index(ip) (ip & 255)
 #define TIMER_OFF -1
 
 typedef struct type_client {
 	t_session *session;
-	int remove_delay;
+	int delayed_remove;
 
 	struct type_client *next;
 } t_client;
@@ -28,37 +29,42 @@ typedef struct type_banned {
 	struct type_banned *next;
 } t_banned;
 
-t_client *clientlist;
-t_banned *banlist;
-pthread_mutex_t client_mutex;
-pthread_mutex_t ban_mutex;
-
-static void insecure_kick_ip(long ip);
+static t_client *client_list[256];
+static pthread_mutex_t client_mutex[256];
+static t_banned *banlist;
+static pthread_mutex_t ban_mutex;
 
 /* Initialize this module.
  */
-void init_clientmodule(void) {
-	clientlist = NULL;
+void init_client_module(void) {
+	int i;
+
+	for (i = 0; i < 256; i++) {
+		client_list[i] = NULL;
+		pthread_mutex_init(&client_mutex[i], NULL);
+	}
 	banlist = NULL;
-	pthread_mutex_init(&client_mutex, NULL);
 	pthread_mutex_init(&ban_mutex, NULL);
 }
 
-/* Add the session record of a client to the clientlist.
+/* Add the session record of a client to the client_list.
  */
 int add_client(t_session *session) {
 	t_client *new;
+	int i;
 
 	if ((new = (t_client*)malloc(sizeof(t_client))) != NULL) {
 		new->session = session;
-		new->remove_delay = -1;
+		new->delayed_remove = TIMER_OFF;
 
-		pthread_mutex_lock(&client_mutex);
+		i = ip_index(session->ip_address);
 
-		new->next = clientlist;
-		clientlist = new;
+		pthread_mutex_lock(&client_mutex[i]);
 
-		pthread_mutex_unlock(&client_mutex);
+		new->next = client_list[i];
+		client_list[i] = new;
+
+		pthread_mutex_unlock(&client_mutex[i]);
 
 		return 0;
 	} else {
@@ -70,73 +76,82 @@ int add_client(t_session *session) {
  */
 void mark_client_for_removal(t_session *session, int delay) {
 	t_client *list;
+	int i;
 
-	pthread_mutex_lock(&client_mutex);
+	i = ip_index(session->ip_address);
 
-	list = clientlist;
+	pthread_mutex_lock(&client_mutex[i]);
+
+	list = client_list[i];
 	while (list != NULL) {
 		if (list->session == session) {
 			close_socket(list->session);
-			list->remove_delay = delay - 1;
+			list->delayed_remove = delay - 1;
 			break;
 		}
 		list = list->next;
 	}
 
-	pthread_mutex_unlock(&client_mutex);
+	pthread_mutex_unlock(&client_mutex[i]);
 }
 
-/* Check the remove_delay timers and remove client
+/* Check the delayed_remove timers and remove client
  * when timer has reached 0
  */
-void check_remove_delay_timers(void) {
-	t_client *list, *prev = NULL, *next;
+void check_delayed_remove_timers(void) {
+	t_client *client, *prev = NULL, *next;
+	int i;
 
-	pthread_mutex_lock(&client_mutex);
+	for (i = 0; i < 256; i++) {
+		pthread_mutex_lock(&client_mutex[i]);
 
-	list = clientlist;
-	while (list != NULL) {
-		next = list->next;
-		switch (list->remove_delay) {
-			case -1:
-				break;
-			case 0:
-				free(list->session);
-				free(list);
-				list = prev;
-				if (prev == NULL) {
-					clientlist = next;
-				} else {
-					prev->next = next;
-				}
-				break;
-			default:
-				list->remove_delay--;
+		client = client_list[i];
+		while (client != NULL) {
+			next = client->next;
+			switch (client->delayed_remove) {
+				case TIMER_OFF:
+					break;
+				case 0:
+					free(client->session);
+					free(client);
+					client = prev;
+					if (prev == NULL) {
+						client_list[i] = next;
+					} else {
+						prev->next = next;
+					}
+					break;
+				default:
+					client->delayed_remove--;
+			}
+			prev = client;
+			client = next;
 		}
-		prev = list;
-		list = next;
-	}
 
-	pthread_mutex_unlock(&client_mutex);
+		pthread_mutex_unlock(&client_mutex[i]);
+	}
 }
 
-/* Remove a client from the clientlist.
+/* Remove a client from the client_list.
  */
 void remove_client(t_session *session, bool free_session) {
-	t_client *deadone = NULL, *list;
+	t_client *to_be_removed = NULL, *list;
+	int i;
 
-	pthread_mutex_lock(&client_mutex);
+	i = ip_index(session->ip_address);
 
-	if (clientlist != NULL) {
-		if (clientlist->session == session) {
-			deadone = clientlist;
-			clientlist = clientlist->next;
+	pthread_mutex_lock(&client_mutex[i]);
+
+	if (client_list != NULL) {
+		if (client_list[i]->session == session) {
+			to_be_removed = client_list[i];
+			client_list[i] = client_list[i]->next;
 		} else {
-			list = clientlist;
+			list = client_list[i];
 			while (list->next != NULL) {
 				if (list->next->session == session) {
-					deadone = list->next;
-					list->next = deadone->next;
+					to_be_removed = list->next;
+					list->next = to_be_removed->next;
 					break;
 				}
 				list = list->next;
@@ -144,14 +159,14 @@ void remove_client(t_session *session, bool free_session) {
 		}
 	}
 
-	pthread_mutex_unlock(&client_mutex);
+	pthread_mutex_unlock(&client_mutex[i]);
 
-	if (deadone != NULL) {
+	if (to_be_removed != NULL) {
 		if (free_session) {
-			close_socket(deadone->session);
-			free(deadone->session);
+			close_socket(to_be_removed->session);
+			free(to_be_removed->session);
 		}
-		free(deadone);
+		free(to_be_removed);
 	}
 }
 
@@ -160,7 +175,7 @@ void remove_client(t_session *session, bool free_session) {
 int connection_allowed(long ip, int maxperip, int maxtotal) {
 	bool banned = false;
 	t_banned *ban;
-	int perip = 0, total = 0;
+	int perip = 0, total = 0, i;
 	t_client *client;
 
 	pthread_mutex_lock(&ban_mutex);
@@ -181,18 +196,22 @@ int connection_allowed(long ip, int maxperip, int maxtotal) {
 		return ca_BANNED;
 	}
 
-	pthread_mutex_lock(&client_mutex);
+	for (i = 0; i < 256; i++) {
+		pthread_mutex_lock(&client_mutex[i]);
 
-	client = clientlist;
-	while (client != NULL) {
-		if (client->session->ip_address == ip) {
-			perip++;
+		client = client_list[i];
+		while (client != NULL) {
+			if (client->session->ip_address == ip) {
+				perip++;
+			}
+			if (client->delayed_remove == TIMER_OFF) {
+				total++;
+			}
+			client = client->next;
 		}
-		total++;
-		client = client->next;
-	}
 
-	pthread_mutex_unlock(&client_mutex);
+		pthread_mutex_unlock(&client_mutex[i]);
+	}
 
 	if (perip < maxperip) {
 		if (total < maxtotal) {
@@ -210,20 +229,24 @@ int connection_allowed(long ip, int maxperip, int maxtotal) {
 void disconnect_clients(t_config *config) {
 	t_client *client;
 	t_directory *dir;
-	int max_wait = 30;
+	int max_wait = 30, i;
 
-	pthread_mutex_lock(&client_mutex);
+	for (i = 0; i < 256; i++) {
+		pthread_mutex_lock(&client_mutex[i]);
 
-	client = clientlist;
-	while (client != NULL) {
-		client->session->force_quit = true;
-		client = client->next;
+		client = client_list[i];
+		while (client != NULL) {
+			client->session->force_quit = true;
+			client = client->next;
+		}
+
+		pthread_mutex_unlock(&client_mutex[i]);
 	}
 
-	pthread_mutex_unlock(&client_mutex);
-
-	while ((clientlist != NULL) && (max_wait-- > 0)) {
-		usleep(100000);
+	for (i = 0; i < 256; i++) {
+		while ((client_list[i] != NULL) && (max_wait-- > 0)) {
+			usleep(100000);
+		}
 	}
 
 	dir = config->directory;
@@ -233,70 +256,70 @@ void disconnect_clients(t_config *config) {
 	}
 }
 
-/* Check the request sending speed of the clients.
+/* Kick an IP address.
  */
-void check_flooding(t_config *config) {
+void kick_ip(long ip) {
 	t_client *client;
 
-	if (config->ban_on_flooding > 0) {
-		pthread_mutex_lock(&client_mutex);
+	pthread_mutex_lock(&client_mutex[ip_index(ip)]);
 
-		client = clientlist;
-		while (client != NULL) {
-			if (client->session->flooding_timer > 0) {
-				client->session->flooding_timer--;
-			} else if (client->session->flooding_counter < config->flooding_count) {
-				client->session->flooding_timer = config->flooding_time;
-				client->session->flooding_counter = 0;
-			} else if (ip_allowed(client->session->ip_address, config->banlist_mask)) {
-				ban_ip(client->session->ip_address, config->ban_on_flooding, false);
-				log_warning(config->system_logfile, "Client banned because of flooding", client->session->ip_address);
-				if (config->kick_on_ban) {
-					insecure_kick_ip(client->session->ip_address);
-				} else {
-					close_socket(client->session);
-				}
-			}
-			client = client->next;
+	client = client_list[ip_index(ip)];
+	while (client != NULL) {
+		if (client->session->ip_address == ip) {
+			client->session->force_quit = true;
 		}
-
-		pthread_mutex_unlock(&client_mutex);
+		client = client->next;
 	}
+
+	pthread_mutex_unlock(&client_mutex[ip_index(ip)]);
+}
+
+/* Check if the client is flooding the server with requests
+ */
+bool client_is_flooding(t_session *session) {
+	time_t time_passed;
+
+	time_passed = time(NULL) - session->flooding_timer + 1;
+
+	return ((session->kept_alive * session->config->flooding_time) > (session->config->flooding_count * time_passed));
 }
 
 #ifdef HAVE_COMMAND
 /* Extra client functions
  */
-void print_clientlist(FILE *fp) {
+void print_client_list(FILE *fp) {
 	t_client *client;
 	char *host;
 	unsigned char *ip_address;
+	int i;
 
-	pthread_mutex_lock(&client_mutex);
+	for (i = 0; i < 256; i++) {
+		pthread_mutex_lock(&client_mutex[i]);
 
-	client = clientlist;
-	while (client != NULL) {
-		fprintf(fp, "session id  : %d\n", client->session->client_id);
-		ip_address = (unsigned char*)&(client->session->ip_address);
-		fprintf(fp, "IP-address  : %hhu.%hhu.%hhu.%hhu\n", ip_address[0], ip_address[1], ip_address[2], ip_address[3]);
-		fprintf(fp, "socket      : %d\n", client->session->client_socket);
-		fprintf(fp, "Kept alive  : %d\n", client->session->kept_alive);
-		if (client->session->remote_user != NULL) {
-		fprintf(fp, "Remote user : %s\n", client->session->remote_user);
-		}
-		if (client->session->file_on_disk != NULL) {
-			fprintf(fp, "File on disk: %s\n", client->session->file_on_disk);
-			if ((host = get_headerfield("Host:", client->session->headerfields)) != NULL) {
-				fprintf(fp, "Host        : %s\n", host);
+		client = client_list[i];
+		while (client != NULL) {
+			fprintf(fp, "session id  : %d\n", client->session->client_id);
+			ip_address = (unsigned char*)&(client->session->ip_address);
+			fprintf(fp, "IP-address  : %hhu.%hhu.%hhu.%hhu\n", ip_address[0], ip_address[1], ip_address[2], ip_address[3]);
+			fprintf(fp, "socket      : %d\n", client->session->client_socket);
+			fprintf(fp, "Kept alive  : %d\n", client->session->kept_alive);
+			if (client->session->remote_user != NULL) {
+			fprintf(fp, "Remote user : %s\n", client->session->remote_user);
+			}
+			if (client->session->file_on_disk != NULL) {
+				fprintf(fp, "File on disk: %s\n", client->session->file_on_disk);
+				if ((host = get_headerfield("Host:", client->session->headerfields)) != NULL) {
+					fprintf(fp, "Host        : %s\n", host);
+				}
+			}
+			client = client->next;
+			if (client != NULL) {
+				fprintf(fp, "---\n");
 			}
 		}
-		client = client->next;
-		if (client != NULL) {
-			fprintf(fp, "---\n");
-		}
-	}
 
-	pthread_mutex_unlock(&client_mutex);
+		pthread_mutex_unlock(&client_mutex[i]);
+	}
 }
 #endif
 
@@ -304,40 +327,22 @@ void print_clientlist(FILE *fp) {
  */
 void kick_client(int id) {
 	t_client *client;
-	
-	pthread_mutex_lock(&client_mutex);
+	int i;
 
-	client = clientlist;
-	while (client != NULL) {
-		if (client->session->client_id == id) {
-			client->session->force_quit = true;
-			break;
+	for (i = 0; i < 256; i++) {
+		pthread_mutex_lock(&client_mutex[i]);
+
+		client = client_list[i];
+		while (client != NULL) {
+			if (client->session->client_id == id) {
+				client->session->force_quit = true;
+				break;
+			}
+			client = client->next;
 		}
-		client = client->next;
+
+		pthread_mutex_unlock(&client_mutex[i]);
 	}
-
-	pthread_mutex_unlock(&client_mutex);
-}
-
-/* Kick an IP address.
- */
-static void insecure_kick_ip(long ip) {
-	t_client *client;
-
-	client = clientlist;
-	while (client != NULL) {
-		if (client->session->ip_address == ip) {
-			client->session->force_quit = true;
-		}
-		client = client->next;
-	}
-}
-void kick_ip(long ip) {
-	pthread_mutex_lock(&client_mutex);
-
-	insecure_kick_ip(ip);
-
-	pthread_mutex_unlock(&client_mutex);
 }
 
 /* IP ban functions
