@@ -1,6 +1,7 @@
 #ifndef _CLIENT_H
 #define _CLIENT_H
 
+#include <stdbool.h>
 #include "serverconfig.h"
 #include "session.h"
 
@@ -8,13 +9,14 @@
 #define ca_TOOMUCH_TOTAL  -2
 #define ca_BANNED         -3
 
-void init_clientmodule(void);
+void init_client_module(void);
 int  add_client(t_session *session);
 void mark_client_for_removal(t_session *session, int delay);
-void check_remove_delay_timers(void);
+void check_delayed_remove_timers(void);
 void remove_client(t_session *session, bool free_session);
 int  connection_allowed(long ip, int maxperip, int maxtotal);
 void disconnect_clients(t_config *config);
+bool client_is_flooding(t_session *session);
 void check_flooding(t_config *config);
 
 #ifdef HAVE_COMMAND
