@@ -18,21 +18,20 @@
 #ifdef HAVE_RPCSVC_CRYPT_H
 #include <rpcsvc/crypt.h>
 #endif
-#include "command.h"
 #include "libstr.h"
+#include "command.h"
 #include "client.h"
-#include "session.h"
 
 #define MAX_IDLE_TIME 30
 
-t_admin *adminlist, *loop, *next;
+static t_admin *adminlist, *loop, *next;
 static char *prompt = "cmd> ";
 static char *unknown_cmd = "Unknown command. Type 'help' for the commandlist.\n";
 
-volatile int cl_lock = 0;
-pthread_mutex_t command_mutex;
+static volatile int cl_lock = 0;
+static pthread_mutex_t command_mutex;
 
-unsigned long clients_served = 0, req_files = 0, req_cgi = 0, req_index = 0;
+static unsigned long clients_served = 0, req_files = 0, req_cgi = 0, req_index = 0;
 
 /* A client needs to change a session record. Admins must stay out.
  */
@@ -74,7 +73,7 @@ void CC_admin_unlock(void) {
 
 /* Initialize the CommandChannel
  */
-void init_commandmodule(void) {
+void init_command_module(void) {
     adminlist = NULL;
 	loop = NULL;
 	next = NULL;
@@ -274,10 +273,10 @@ int handle_admin(t_admin *admin, t_config *config) {
 					}
 				}
 			} else {
-				memcpy(salt, config->command_port->name, 2);
+				memcpy(salt, config->command_port->binding_id, 2);
 				salt[2] = '\0';
 				encrypted = crypt(remove_spaces(line), salt);
-				if ((admin->allowed = (strcmp(encrypted, config->command_port->name) == 0)) == false) {
+				if ((admin->allowed = (strcmp(encrypted, config->command_port->binding_id) == 0)) == false) {
 					retval = cc_DISCONNECT;
 				}
 			}
