@@ -1,9 +1,6 @@
 #ifndef _COMMAND_H
 #define _COMMAND_H
 
-#ifdef HAVE_COMMAND
-
-#include "config.h"
 #include "serverconfig.h"
 
 #define cc_DISCONNECT    -1
@@ -24,7 +21,7 @@ void CC_client_unlock(void);
 void CC_admin_lock(void);
 void CC_admin_unlock(void);
 
-void init_commandmodule(void);
+void init_command_module(void);
 int add_admin(int sock);
 void remove_admin(int sock);
 void check_adminlist(void);
@@ -34,5 +31,3 @@ int handle_admin(t_admin *admin, t_config *config);
 void disconnect_admins(void);
 
 #endif
-
-#endif
