@@ -26,7 +26,7 @@
 #include "libfs.h"
 #include "liblist.h"
 #include "libstr.h"
-#include "libcgi.h"
+#include "cgi.h"
 #include "userconfig.h"
 #include "session.h"
 #include "httpauth.h"
@@ -34,6 +34,9 @@
 #include "client.h"
 #include "target.h"
 #include "log.h"
+#ifndef HAVE_SETENV
+#include "envir.h"
+#endif
 #ifdef HAVE_COMMAND
 #include "command.h"
 #endif
@@ -43,13 +46,14 @@
 #ifdef HAVE_PLUGIN
 #include "plugin.h"
 #endif
+#ifdef HAVE_CACHE
+#include "cache.h"
+#endif
 
 #define FILENAME_BUFFER_SIZE   256
 #define REQUEST_BUFFER_CHUNK  4096
 #define MAIN_LOOP_TIMER          1
 
-#define GENERAL_ERROR           -1
-
 #define fe_MAX_REQUESTSIZE      -2
 #define fe_TIMEOUT              -3
 #define fe_CLIENT_DISCONNECTED  -4
@@ -63,7 +67,6 @@
 
 volatile int received_signal = rs_NONE;
 
-char *pidfile                = PIDFILE_DIR"/hiawatha.pid";
 char *default_configdir      = CONFIG_DIR;
 char *main_configfile        = "httpd.conf";
 
@@ -131,14 +134,18 @@ int handle_error(t_session *session) {
 	strcpy(session->file_on_disk + length, session->host->error_handler);
 
 	if ((session->is_cgi_script = in_charlist(file_extension(session->file_on_disk), &(session->config->cgi_extension))) == false) {
-		cgi = session->config->cgi_handler;
-		while (cgi != NULL) {
-			if (in_charlist(file_extension(session->file_on_disk), &(cgi->extension))) {
-				session->is_cgi_script = true;
-				session->cgi_handler = cgi->handler;
-				break;
+		if ((session->fcgi_server = fcgi_server_match(session->config->fcgi_server, &(session->host->fast_cgi), session->file_on_disk)) != NULL) {
+			session->is_cgi_script = true;
+		} else {
+			cgi = session->config->cgi_handler;
+			while (cgi != NULL) {
+				if (in_charlist(file_extension(session->file_on_disk), &(cgi->extension))) {
+					session->is_cgi_script = true;
+					session->cgi_handler = cgi->handler;
+					break;
+				}
+				cgi = cgi->next;
 			}
-			cgi = cgi->next;
 		}
 	}
 
@@ -146,7 +153,7 @@ int handle_error(t_session *session) {
 		if (session->host->return_code != -1) {
 			session->return_code = session->host->return_code;
 		}
-		retval = run_cgi_program(session);
+		retval = run_script(session);
 	} else {
 		retval = send_file(session);
 	}
@@ -161,14 +168,18 @@ int handle_request(t_session *session) {
 	t_cgi_handler *cgi;
 
 	if ((session->is_cgi_script = in_charlist(file_extension(session->file_on_disk), &(session->config->cgi_extension))) == false) {
-		cgi = session->config->cgi_handler;
-		while (cgi != NULL) {
-			if (in_charlist(file_extension(session->file_on_disk), &(cgi->extension))) {
-				session->cgi_handler = cgi->handler;
-				session->is_cgi_script = true;
-				break;
+		if ((session->fcgi_server = fcgi_server_match(session->config->fcgi_server, &(session->host->fast_cgi), session->file_on_disk)) != NULL) {
+			session->is_cgi_script = true;
+		} else {
+			cgi = session->config->cgi_handler;
+			while (cgi != NULL) {
+				if (in_charlist(file_extension(session->file_on_disk), &(cgi->extension))) {
+					session->cgi_handler = cgi->handler;
+					session->is_cgi_script = true;
+					break;
+				}
+				cgi = cgi->next;
 			}
-			cgi = cgi->next;
 		}
 	}
 
@@ -178,7 +189,7 @@ int handle_request(t_session *session) {
 	if ((strcmp(session->method, "GET") == 0) || session->head_request) {
 		if (session->is_cgi_script) {
 			session->body = NULL;
-			retval = run_cgi_program(session);
+			retval = run_script(session);
 		} else {
 			retval = send_file(session);
 		}
@@ -189,7 +200,7 @@ int handle_request(t_session *session) {
 		}
 	} else if (strcmp(session->method, "POST") == 0) {
 		if (session->is_cgi_script) {
-			retval = run_cgi_program(session);
+			retval = run_script(session);
 		} else {
 			retval = 405;
 		}
@@ -246,7 +257,7 @@ int fetch_request(t_session *session) {
 			}
 			if (header_length != -1) {
 				if (content_length == -1) {
-					if ((strstart = str_search(session->request, hs_conlen)) != NULL) {
+					if ((strstart = strcasestr(session->request, hs_conlen)) != NULL) {
 						strstart += 16;
 						if ((strend = strstr(strstart, "\r\n")) != NULL) {
 							*strend = '\0';
@@ -405,7 +416,7 @@ int parse_request(t_session *session, int total_bytes) {
 	switch (*str_end) {
 		case '0':
 			if ((conn != NULL) && (session->kept_alive < session->binding->max_keepalive)) {
-				if (str_compare(conn, "keep-alive") == 0) {
+				if (strcasecmp(conn, "keep-alive") == 0) {
 					session->keep_alive = true;
 				}
 			}
@@ -530,9 +541,11 @@ t_access allow_client(t_session *session) {
 		return allow;
 	} else if (parse_ip(x_forwarded_for, &forwarded_ip) == -1) {
 		return allow;
-	} else {
-		return ip_allowed(forwarded_ip, session->host->access_list);
+	} else if (ip_allowed(forwarded_ip, session->host->access_list) == deny) {
+		return deny;
 	}
+
+	return unknown;
 }
 
 /* Serve the client that connected to the webserver
@@ -560,42 +573,6 @@ int serve_client(t_session *session) {
 		}
 	}
 
-#ifdef HAVE_SSL
-	if (session->host->require_ssl && (session->binding->use_ssl == false)) {
-		session->cause_of_301 = require_ssl;
-		return 301;
-	}
-#endif
-
-	if (valid_uri(session->uri) == false) {
-		return 404;
-	} else if (client_is_rejected_bot(session)) {
-		set_403_reason(session, fb_DENYBOT);
-		return 403;
-	}
-
-#ifdef HAVE_PLUGIN
-	if (is_plugin_uri(session)) {
-		switch (allow_client(session)) {
-			case deny:
-				set_403_reason(session, fb_ACCESSLIST);
-				return 403;
-				break;
-			case allow:
-				set_403_reason(session, fb_ACCESSLIST);
-				break;
-			case pwd:
-				set_403_reason(session, fb_ACCESSLIST);
-			case unknown:
-				set_403_reason(session, fb_ACCESSLIST);
-				if (http_authentication_oke(session) == false) {
-					return 401;
-				}
-				break;
-		}
-		return handle_plugin(session);
-	}
-#endif
 	if ((result = uri_to_path(session)) != 200) {
 		return result;
 	}
@@ -609,7 +586,7 @@ int serve_client(t_session *session) {
 				length = search - dir_config + 1;
 				if ((dir = (char*)malloc(length + 16)) != NULL) {
 					memcpy(dir, dir_config, length);
-					strcpy(dir + length, ".hiawatha");
+					memcpy(dir + length, ".hiawatha\0", 10);
 					if (duplicate_host(session)) {
 						if (read_user_configfile(dir, session->host) == -1) {
 							result = 500;
@@ -638,6 +615,40 @@ int serve_client(t_session *session) {
 		return 500;
 	}
 
+#ifdef HAVE_SSL
+	if (session->host->require_ssl && (session->binding->use_ssl == false)) {
+		session->cause_of_301 = require_ssl;
+		return 301;
+	}
+#endif
+
+	if (valid_uri(session->uri) == false) {
+		return 404;
+	} else if (client_is_rejected_bot(session)) {
+		set_403_reason(session, fb_DENYBOT);
+		return 403;
+	}
+
+#ifdef HAVE_PLUGIN
+	if (is_plugin_uri(session)) {
+		switch (allow_client(session)) {
+			case deny:
+				set_403_reason(session, fb_ACCESSLIST);
+				return 403;
+				break;
+			case allow:
+				break;
+			case pwd:
+			case unknown:
+				if (http_authentication_oke(session) == false) {
+					return 401;
+				}
+				break;
+		}
+		return handle_plugin(session);
+	}
+#endif
+
 	switch (is_directory(session->file_on_disk)) {
 		case error:
 			return 500;
@@ -809,6 +820,16 @@ void connection_handler(t_session *session) {
 #ifdef HAVE_COMMAND
 			CC_client_unlock();
 #endif
+
+			if (session->keep_alive && (session->config->ban_on_flooding > 0)) {
+				if (client_is_flooding(session)) {
+					if (ip_allowed(session->ip_address, session->config->banlist_mask)) {
+						ban_ip(session->ip_address, session->config->ban_on_flooding, session->config->kick_on_ban);
+						log_warning(session->config->system_logfile, "Client banned because of flooding", session->ip_address);
+						session->keep_alive = false;
+					}
+				}
+			}
 		} while (session->keep_alive);
 
 		close_socket(session);
@@ -816,6 +837,7 @@ void connection_handler(t_session *session) {
 		close(session->client_socket);
 	}
 	log_close(session->config->first_host);
+	//destroy_session(session);
 
 #ifdef HAVE_COMMAND
 	CC_client_lock();
@@ -892,6 +914,7 @@ int bind_socket(t_binding *binding) {
 
 	while (binding != NULL) {
 		if ((binding->socket = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
+			perror("socket()");
 			return -1;
 		}
 
@@ -902,16 +925,16 @@ int bind_socket(t_binding *binding) {
 
 		one = 1;
 		if (setsockopt(binding->socket, SOL_SOCKET, SO_REUSEADDR, (void*)&one, sizeof(int)) == -1) {
-			return -1;
+			perror("setsockopt(SOL_SOCKET, SO_REUSEADDR)");
 		}
 		one = 1;
 		if (setsockopt(binding->socket, IPPROTO_TCP, TCP_NODELAY, (void*)&one, sizeof(int)) == -1) {
-			return -1;
+			perror("setsockopt(IPPROTO_TCP, TCP_NODELAY)");
 		}
 
 		if (bind(binding->socket, (struct sockaddr*)&saddr, sizeof(struct sockaddr)) == -1) {
 			ip_address = (char*)&(binding->interface);
-			printf("\nError binding %hhu.%hhu.%hhu.%hhu:%d\n", ip_address[0], ip_address[1], ip_address[2], ip_address[3], binding->port);
+			printf("Error binding %hhu.%hhu.%hhu.%hhu:%d\n", ip_address[0], ip_address[1], ip_address[2], ip_address[3], binding->port);
 			return -1;
 		}
 		binding = binding->next;
@@ -960,9 +983,9 @@ int accept_connection(t_binding *binding, t_config *config) {
 			}
 		}
 
-		pthread_attr_init(&child_attr);
-		pthread_attr_setdetachstate(&child_attr, PTHREAD_CREATE_DETACHED);
 		if (add_client(session) == 0) {
+			pthread_attr_init(&child_attr);
+			pthread_attr_setdetachstate(&child_attr, PTHREAD_CREATE_DETACHED);
 			if (pthread_create(&child_thread, &child_attr, (void*)connection_handler, (void*)session) == 0) {
 				kick_client = false;
 			} else {
@@ -997,15 +1020,35 @@ int accept_connection(t_binding *binding, t_config *config) {
 	return 0;
 }
 
+#ifndef HAVE_CLEARENV
+void clearenv(void) {
+	extern char **environ;
+	char *eq, key[256];
+	int len;
+
+	while (*environ != NULL) {
+		if ((eq = strchr(*environ, '=')) != NULL) {
+			if ((len = eq - *environ) < 256) {
+				memcpy(key, *environ, len);
+				key[len] = '\0';
+				unsetenv(key);
+			}
+		}
+		environ++;
+	}
+}
+#endif
+
 /* Run the Hiawatha webserver.
  */
-void run_server(char *configdir) {
+int run_server(char *configdir, bool daemon) {
 	int                quit = 0, result, highest_fd;
 #ifdef HAVE_COMMAND
 	t_admin            *admin;
 	struct sockaddr_in caddr;
 	int                size, admin_socket;
 #endif
+	pid_t              pid;
 	t_binding          *binding;
 	FILE               *fp;
 	t_config           *config;
@@ -1022,21 +1065,21 @@ void run_server(char *configdir) {
 	}
 	if (result != 0) {
 		print_configerror(main_configfile, result);
-		exit(-1);
+		return -1;
 	}
 	if (read_mimetypes(config->mimetype_config, &(config->mimetype)) == -1) {
-		printf("\nWarning: can't read mimetype configuration.\n");
-		exit(-1);
+		printf("Error while reading mimetype configuration.\n");
+		return -1;
 	}
 	if (read_throttleconfig(config->throttle_config, &(config->throttle)) == -1) {
-		//printf("\nWarning: can't read throttle configuration.\n");
-		//exit(-1);
+		//printf("Error while reading throttle configuration.\n");
+		//return -1
 	}
 
 	/* Bind Serverport
 	 */
 	if (bind_socket(config->binding) == -1) {
-		return;
+		return -1;
 	}
 
 #ifdef HAVE_SSL
@@ -1045,8 +1088,8 @@ void run_server(char *configdir) {
 	while (binding != NULL) {
 		if (binding->use_ssl) {
 			if ((binding->ssl_context = ssl_binding(binding->server_key, "")) == NULL) {
-				printf("\nWarning: error while reading serverkey!\n");
-				return;
+				printf("Error while reading keyfile %s!\n", binding->server_key);
+				return -1;
 			}
 		}
 		binding = binding->next;
@@ -1055,7 +1098,7 @@ void run_server(char *configdir) {
 
 #ifdef HAVE_COMMAND
 	if (bind_socket(config->command_port) == -1) {
-		return;
+		return -1;
 	}
 #endif
 
@@ -1067,20 +1110,38 @@ void run_server(char *configdir) {
 			}
 			break;
 		default:
-			printf("\nWarning: plugin initialization failed.\n");
-			exit(-1);
+			printf("Error while initializing plugin.\n");
+			return -1;
 	}
 #endif
 
-	if ((fp = fopen(pidfile, "w")) != NULL) {
-		fprintf(fp, "%d\n", getpid());
-		fclose(fp);
-	}
 	touch_logfiles(config, config->server_root);
 	tzset();
-#ifdef HAVE_CLEARENV
 	clearenv();
-#endif
+
+	/* Become a daemon
+	 */
+	if (daemon) {
+		switch (pid = fork()) {
+			case -1:
+				perror("fork()");
+				return -1;
+			case 0:
+				if (setsid() == -1) {
+					perror("setsid()");
+					return -1;
+				}
+				break;
+			default:
+				if ((fp = fopen(config->pidfile, "w")) != NULL) {
+					fprintf(fp, "%d\n", pid);
+					fclose(fp);
+				} else {
+					printf("Warning: can't write PID file %s.\n", config->pidfile);
+				}
+				return 0;
+		}
+	}
 
 	/* Security
 	 */
@@ -1091,8 +1152,8 @@ void run_server(char *configdir) {
 					break;
 				}
 			}
-			printf("\nWarning: can't chroot to %s!\n", config->server_root);
-			exit(-1);
+			printf("\nError while changing root to %s!\n", config->server_root);
+			return -1;
 		} while (false);
 	} else {
 		chdir("/");
@@ -1106,25 +1167,29 @@ void run_server(char *configdir) {
 				}
 			}
 		}
-		printf("\nWarning: can't change uid/gid!\n");
-		exit(-1);
+		printf("\nError while changing uid/gid!\n");
+		return -1;
 	} while (false);
 
-	log_string(config->system_logfile, "Hiawatha v"VERSION" started");
-
 	signal(SIGTERM, TERM_handler);
 	signal(SIGUSR1, USR1_handler);
 	signal(SIGUSR2, USR2_handler);
 
 	binding = config->binding;
 	while (binding != NULL) {
-		listen(binding->socket, 16);
+		if (listen(binding->socket, 16) == -1) {
+			perror("listen()");
+			return -1;
+		}
 		binding = binding->next;
 	}
 #ifdef HAVE_COMMAND
 	binding = config->command_port;
 	while (binding != NULL) {
-		listen(binding->socket, 1);
+		if (listen(binding->socket, 1) == -1) {
+			perror("listen()");
+			return -1;
+		}
 		binding = binding->next;
 	}
 #endif
@@ -1132,11 +1197,17 @@ void run_server(char *configdir) {
 	select_timeout.tv_sec = MAIN_LOOP_TIMER;
 	select_timeout.tv_usec = 0;
 
-	init_logmodule();
-	init_clientmodule();
-#ifdef HAVE_COMMAND
-	init_commandmodule();
+	init_log_module();
+	init_client_module();
+	init_load_balancer(config->fcgi_server);
+#ifdef HAVE_CACHE
+	init_cache_module();
 #endif
+#ifdef HAVE_COMMAND
+	init_command_module();
+#endif
+
+	log_string(config->system_logfile, "Hiawatha v"VERSION" started");
 
 	do {
 #ifdef HAVE_COMMAND
@@ -1159,10 +1230,13 @@ void run_server(char *configdir) {
 				select_timeout.tv_usec = 0;
 
 				check_banlist(config);
-				check_flooding(config);
 				if (config->reconnect_delay > 0) {
-					check_remove_delay_timers();
+					check_delayed_remove_timers();
 				}
+				check_load_balancer(config);
+#ifdef HAVE_CACHE
+				check_cache();
+#endif
 #ifdef HAVE_COMMAND
 				check_adminlist();
 #endif
@@ -1284,29 +1358,39 @@ void run_server(char *configdir) {
 #ifdef HAVE_PLUGIN
 	plugin_destroy();
 #endif
+
+	return 0;
+}
+
+void show_help(char *hiabin) {
+	printf("Usage %s: [-c path] [-d] [-h]\n", hiabin);
+	printf("\t-c path: path to where the configrationfiles are located.\n");
+	printf("\t-d: don't fork to the background.\n");
+	printf("\t-h: show this information.\n");
 }
 
 /* Main and stuff...
  */
 int main(int argc, char *argv[]) {
+	int i = 0;
 	char *configdir = default_configdir;
+	bool daemon = true;
 
-	if (argc > 1) {
-		configdir = argv[1];
-	}
-
-	switch (fork()) {
-		case -1:
-			printf("fork() error!\n");
-			break;
-		case 0:
-			if (setsid() == -1) {
-				printf("setsid() error!\n");
-			} else {
-				run_server(configdir);
+	while (++i < argc) {
+		if (strcmp(argv[i], "-c") == 0) {
+			if (++i < argc) {
+				configdir = argv[i];
 			}
-			break;
+		} else if (strcmp(argv[i], "-d") == 0) {
+			daemon = false;
+		} else if (strcmp(argv[i], "-h") == 0) {
+			show_help(argv[0]);
+			return 0;
+		} else {
+			printf("Unknown option\n");
+			return -1;
+		}
 	}
 
-	return 0;
+	return run_server(configdir, daemon);
 }
