@@ -16,7 +16,7 @@ char *months[12] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep
 
 /* Return the size of a file.
  */
-long filesize(char *filename) {
+off_t filesize(char *filename) {
 	struct stat status;
 
 	if (stat(filename, &status) == 0) {
@@ -188,8 +188,7 @@ void touch_logfile(char *dir, char *file, mode_t mode, uid_t uid, gid_t gid) {
 
 	if (dir != NULL) {
 		if ((logfile = (char*)malloc(strlen(dir) + strlen(file) + 1)) != NULL) {
-			strcpy(logfile, dir);
-			strcat(logfile, file);
+			sprintf(logfile, "%s%s", dir, file);
 		}   
 	} else {
 		logfile = file;
@@ -314,7 +313,7 @@ int if_modified_since(int handle, char *datestr) {
 	return -1;
 }
 
-/* Open a file (search in directory where file 'neighbour' is located if not found).
+/* Open a file (searches in directory where file 'neighbour' is located if not found).
  */
 FILE *fopen_neighbour(char *filename, char *mode, char *neighbour) {
 	FILE *fp;
