@@ -17,7 +17,7 @@ typedef struct type_filelist {
 	struct type_filelist *next;
 } t_filelist;
 
-long filesize(char *filename);
+off_t filesize(char *filename);
 enum t_fsbool contains_not_allowed_symlink(char *filename, char *webroot);
 enum t_fsbool is_directory(char *file);
 enum t_fsbool can_execute(char *file, uid_t uid, gid_t gid, t_groups *groups);
