@@ -143,7 +143,7 @@ t_accesslist *parse_accesslist(char *line, bool pwd_allowed, t_accesslist *list)
 				new->access = allow;
 			} else if (strcmp(rule, "deny") == 0) {
 				new->access = deny;
-			} else if ((pwd_allowed) && (strcmp(rule, "pwd") == 0)) {
+			} else if (pwd_allowed && (strcmp(rule, "pwd") == 0)) {
 				new->access = pwd;
 			} else {
 				list = remove_accesslist(list);
