@@ -1,6 +1,8 @@
 #ifndef _LIBLIST_H
 #define _LIBLIST_H
 
+#include <stdbool.h>
+
 typedef enum { deny, allow, pwd, unknown } t_access;
 
 typedef struct type_charlist {
@@ -8,15 +10,6 @@ typedef struct type_charlist {
 	char **item;
 } t_charlist;
 
-typedef struct type_bindlist {
-	unsigned long ip;
-	int port;
-	int socket;
-	char *name;
-
-	struct type_bindlist *next;
-} t_bindlist;
-
 typedef struct type_accesslist {
 	unsigned long ip;
 	int netmask;
@@ -44,10 +37,6 @@ void sfree(void* ptr);
 void init_charlist(t_charlist *list);
 int parse_charlist(char *value, t_charlist *list);
 bool in_charlist(char *item, t_charlist *list);
-void remove_charlist(t_charlist *list);
-
-t_bindlist *parse_bindlist(char *rest, int default_port, t_bindlist *list);
-t_bindlist *remove_bindlist(t_bindlist *list);
 
 int parse_ip(char *line, unsigned long *l_ip);
 t_accesslist *parse_accesslist(char *line, bool pwd_allowed, t_accesslist *list);
