@@ -7,6 +7,11 @@
 #include <assert.h>
 #include <pthread.h>
 #include "libssl.h"
+//#ifndef HAVE_SETENV
+//#include "envir.h"
+//#endif
+
+#define CIPHERS "DH-DSS-AES256-SHA:DH-RSA-AES256-SHA:DHE-DSS-AES256-SHA:DHE-RSA-AES256-SHA:AES256-SHA"
 
 static pthread_mutex_t *locks;
 static int lockCount;
@@ -57,9 +62,10 @@ void sslInit(void) {
 	/* Initialize SSL library's thread locks. */
 	lockCount = CRYPTO_num_locks();
 	if (lockCount) {
-		locks = malloc(lockCount * sizeof(pthread_mutex_t));
-		for (i = 0; i < lockCount; i++) {
-			pthread_mutex_init(&locks[i], NULL);
+		if ((locks = malloc(lockCount * sizeof(pthread_mutex_t))) != NULL) {
+			for (i = 0; i < lockCount; i++) {
+				pthread_mutex_init(&locks[i], NULL);
+			}
 		}
     }
   
@@ -99,6 +105,9 @@ SSL_CTX *ssl_binding(const char* sslKeyFile, const char* sslPassword) {
 		return NULL;
 	}
 
+	SSL_CTX_set_cipher_list(sslContext, CIPHERS);
+//	SSL_CTX_set_verify(sslContext, SSL_VERIFY_PEER | SSL_VERIFY_FAIL_IF_NO_PEER_CERT, 0);
+
 	return sslContext;
 }
 
@@ -122,36 +131,24 @@ bool ssl_accept(int socketFD, SSL **ssl, SSL_CTX *sslContext) {
 		case SSL_ERROR_ZERO_RETURN:
 			fprintf(stderr, "SSL zero return error\n");
 			break;
-			
 		case SSL_ERROR_WANT_READ:
 			fprintf(stderr, "SSL_ERROR_WANT_READ\n");
 			break;
-			
 		case SSL_ERROR_WANT_WRITE:
 			fprintf(stderr, "SSL_ERROR_WANT_WRITE\n");
 			break;
-			
 		case SSL_ERROR_WANT_CONNECT:
 			fprintf(stderr, "SSL_ERROR_WANT_CONNECT\n");
 			break;
-			
-			/* Too new? Too old? OpenSSL docs are the worst!
-		case SSL_ERROR_WANT_ACCEPT:
-			fprintf( stderr, "SSL_ERROR_WANT_ACCEPT\n" );
-			return false;
-			*/ 
 		case SSL_ERROR_WANT_X509_LOOKUP:
 			fprintf(stderr, "SSL_X509_LOOKUP\n");
 			break;
-			
 		case SSL_ERROR_SYSCALL:
 			fprintf(stderr, "SSL_ERROR_SYSCALL\n");
 			break;
-			
 		case SSL_ERROR_SSL:
 			fprintf(stderr, "SSL_ERROR_SSL\n");
 			break;
-			
 		default:
 			fprintf(stderr, "SSL Error unknown!\n");
 			break;
@@ -209,4 +206,21 @@ void ssl_cleanup(SSL_CTX *sslContext) {
 	}
 }
 
+/*
+void ssl_set_environment(SSL *ssl_data) {
+	char data[256];
+	X509 *cert;
+
+	data[255] = '\0';
+	
+	if ((cert = SSL_get_peer_certificate(ssl_data)) != NULL) {
+		X509_NAME_oneline(X509_get_subject_name(cert), data, 255);
+		setenv("SSL_CLIENT_SUBJECT", data, 1);
+
+		X509_NAME_oneline(X509_get_issuer_name(cert), data, 255);
+		setenv("SSL_CLIENT_ISSUER", data, 1);
+	}
+}
+*/
+
 #endif
