@@ -150,8 +150,8 @@ int parse_special_chars(char *line, int length) {
 						line += 2;
 						if ((*line = (char)(high<<4) + low) < 32) {
 							*line = '_';
-							result -= 2;
 						}
+						result -= 2;
 					}
 				}
 			}
@@ -227,7 +227,7 @@ int prevent_XSS(char *line) {
 
 	if (line != NULL) {
 		while (*line != '\0') {
-			if ((*line == '\"') || (*line == '<') || (*line == '>')) {
+			if ((*line == '\"') || (*line == '<') || (*line == '>') || (*line == '\'')) {
 				*line = '_';
 				result += 1;
 			}
@@ -396,83 +396,41 @@ char *file_extension(char *filename) {
 	return NULL;
 }
 
-static char charlower(char c) {
-	if ((c >= 'A') && (c <= 'Z')) {
-		c += 32;
+char *str_replace(char *line, char *from, char *to) {
+	char *pos, *new = NULL;
+	bool replaced = false;
+
+	while ((pos = strstr(line, from)) != NULL) {
+		if ((new = (char*)malloc(strlen(line) - strlen(from) + strlen(to) + 1)) == NULL) {
+			return NULL;
+		}
+
+		*pos = '\0';
+		sprintf(new, "%s%s%s", line, to, pos + strlen(from));
+
+		if (replaced) {
+			free(line);
+		} else {
+			replaced = true;
+		}
+		line = new;
 	}
 
-	return c;
+	return new;
 }
 
-int str_compare(char *str1, char *str2) {
-	while (*str1 != '\0') {
-		if (*str1 != *str2) {
-			if (charlower(*str1) != charlower(*str2)) {
-				return -1;
-			}
-		}
-		str1++;
-		str2++;
-	}
-
-	if (*str2 == '\0') {
-		return 0;
-	} else {
-		return -1;
-	}
-}
-
-int str_ncompare(char *str1, char *str2, int len) {
-	while (len-- > 0) {
-		if (*str1 != *str2) {
-			if (charlower(*str1) != charlower(*str2)) {
-				return -1;
-			}
-		}
-		if (*str1 == '\0') {
-			break;
-		}
-		str1++;
-		str2++;
-	}
-
-	return 0;
-}
-
-char *str_search(char *haystack, char *needle) {
+#ifndef HAVE_STRCASESTR
+char *strcasestr(char *haystack, char *needle) {
 	int i, len, steps;
 
 	len = strlen(needle);
 	steps = strlen(haystack) - len + 1;
 	for (i = 0; i < steps; i++) {
-		if (str_ncompare(haystack + i, needle, len) == 0) {
+		if (strncasecmp(haystack + i, needle, len) == 0) {
 			return haystack + i;
 		}
 	}
-	
+
 	return NULL;
 }
-
-char *str_replace(char *line, char *from, char *to) {
-	char *pos, *new = NULL;
-	bool replaced = false;
-
-	if ((pos = strstr(line, from)) != NULL) {
-		do {
-			new = (char*)malloc(strlen(line) - strlen(from) + strlen(to) + 1);
-			*pos = '\0';
-			strcpy(new, line);
-			strcat(new, to);
-			strcat(new, pos + strlen(from));
-
-			if (replaced) {
-				free(line);
-			} else {
-				replaced = true;
-			}
-			line = new;
-		} while ((pos = strstr(line, from)) != NULL);
-	}
-
-	return new;
-}
+#endif
