@@ -1,3 +1,5 @@
+#include "config.h"
+
 #ifndef _LIBSTR_H
 #define _LIBSTR_H
 
@@ -16,9 +18,9 @@ int prevent_XSS(char *line);
 const char *errorstr(int code);
 bool decode_base64(char *base64);
 char *file_extension(char *filename);
-int str_compare(char *str1, char *str2);
-int str_ncompare(char *str1, char *str2, int len);
-char *str_search(char *haystack, char *needle);
 char *str_replace(char *line, char *from, char *to);
+#ifndef HAVE_STRCASESTR
+char *strcasestr(char *haystack, char *needle);
+#endif
 
 #endif
