@@ -1,3 +1,7 @@
+/* Hiawatha | log.c
+ *
+ * Logging functions
+ */
 #include "config.h"
 #include <stdio.h>
 #include <stdbool.h>
@@ -7,15 +11,16 @@
 #include <sys/file.h>
 #include "log.h"
 
-#define BUFFER_SIZE 2000
+#define BUFFER_SIZE     2000
+#define TIMESTAMP_SIZE    40
+#define IP_ADDRESS_SIZE   20
 
 pthread_mutex_t accesslog_mutex;
 
-void init_logmodule(void) {
+void init_log_module(void) {
 	pthread_mutex_init(&accesslog_mutex, NULL);
 }
 
-
 /* Write a timestamp to a logfile.
  */
 static void print_timestamp(char *str) {
@@ -24,7 +29,8 @@ static void print_timestamp(char *str) {
 
 	time(&t);
 	s = localtime(&t);
-	strftime(str, 60, "%a %d %b %Y %X|", s);
+	str[TIMESTAMP_SIZE - 1] = '\0';
+	strftime(str, TIMESTAMP_SIZE - 1, "%a %d %b %Y %X|", s);
 }
 
 /* Write an IP address to a logfile.
@@ -33,7 +39,8 @@ static void print_ip_address(char *str, long ip_address) {
 	unsigned char *ip;
 
 	ip = (unsigned char*)&ip_address;
-	sprintf(str, "%hhu.%hhu.%hhu.%hhu|", ip[0], ip[1], ip[2], ip[3]);
+	str[IP_ADDRESS_SIZE - 1] = '\0';
+	snprintf(str, IP_ADDRESS_SIZE - 1, "%hhu.%hhu.%hhu.%hhu|", ip[0], ip[1], ip[2], ip[3]);
 }
 
 static char *secure_string(char *str) {
@@ -55,7 +62,7 @@ static char *secure_string(char *str) {
  */
 void log_string(char *logfile, char *data) {
 	FILE *fp;
-	char str[60];
+	char str[TIMESTAMP_SIZE];
 
 	if ((fp = fopen(logfile, "a")) != NULL) {
 		print_timestamp(str);
@@ -66,9 +73,9 @@ void log_string(char *logfile, char *data) {
 
 /* Log a CGI error.
  */
-void log_CGI_error(char *logfile, char *CGI_file, char *error) {
+void log_cgi_error(char *logfile, char *CGI_file, char *error) {
 	FILE *fp;
-	char *c, str[60];
+	char *c, str[TIMESTAMP_SIZE];
 
 	c = error;
 	while (*c != '\0') {
@@ -89,7 +96,7 @@ void log_CGI_error(char *logfile, char *CGI_file, char *error) {
  */
 void log_warning(char *logfile, char *data, long ip_address) {
 	FILE *fp;
-	char str[60];
+	char str[TIMESTAMP_SIZE + IP_ADDRESS_SIZE];
 
 	if ((fp = fopen(logfile, "a")) != NULL) {
 		print_ip_address(str, ip_address);
@@ -102,7 +109,7 @@ void log_warning(char *logfile, char *data, long ip_address) {
 /* Log a HTTP request.
  */
 void log_request(t_session *session, int code) {
-	char rcode[4], *user;
+	char rcode[5], *user;
 	t_headerfield *headerfield;
 	bool should_log;
 	char str[BUFFER_SIZE + 1];
@@ -111,7 +118,8 @@ void log_request(t_session *session, int code) {
 	if (ip_allowed(session->ip_address, session->config->logfile_mask) != deny) {
 		str[BUFFER_SIZE] = '\0';
 
-		sprintf(rcode, "%d", code);
+		rcode[4] = '\0';
+		snprintf(rcode, 4, "%d", code);
 		if ((user = session->remote_user) == NULL) {
 			user = "";
 		}
@@ -168,7 +176,7 @@ void log_request(t_session *session, int code) {
  */
 void log_garbage(t_session *session) {
 	FILE *fp;
-	char str[100];
+	char str[TIMESTAMP_SIZE + IP_ADDRESS_SIZE];
 
 	if ((session->config->garbage_logfile != NULL) && (session->request != NULL)) {
 		if ((fp = fopen(session->config->garbage_logfile, "a")) != NULL) {
@@ -184,7 +192,7 @@ void log_garbage(t_session *session) {
  */
 void log_unban(char *logfile, unsigned long ip_address, unsigned long connect_attempts) {
 	FILE *fp;
-	char str[60];
+	char str[TIMESTAMP_SIZE + IP_ADDRESS_SIZE];
 
 	if ((fp = fopen(logfile, "a")) != NULL) {
 		print_ip_address(str, ip_address);
@@ -208,17 +216,14 @@ void log_close(t_host *host) {
 	pthread_mutex_unlock(&accesslog_mutex);
 }
 
-#ifdef DEBUG
-void log_error(char *logfile, int code) {
+void log_error(t_session *session, char *mesg) {
 	FILE *fp;
-	char str[1024];
+	char str[TIMESTAMP_SIZE + IP_ADDRESS_SIZE];
 
-	if ((fp = fopen(logfile, "a")) != NULL) {
+	if ((fp = fopen(session->config->system_logfile, "a")) != NULL) {
 		print_timestamp(str);
-		sprintf(str + strlen(str), "ERROR: ");
-		strerror_r(code, str + strlen(str), 255);
-		fprintf(fp, "%s\n", str);
+		print_timestamp(str + strlen(str));
+		fprintf(fp, "%serror:%s\n", str, mesg);
 		fclose(fp);
 	}
 }
-#endif
