@@ -3,16 +3,14 @@
 
 #include "session.h"
 
-void init_logmodule(void);
+void init_log_module(void);
 void log_string(char *logfile, char *data);
-void log_CGI_error(char *logfile, char *CGI_file, char *error);
+void log_cgi_error(char *logfile, char *cgi_file, char *error);
 void log_warning(char *logfile, char *req, long ip_address);
 void log_request(t_session *session, int code);
 void log_garbage(t_session *session);
 void log_unban(char *logfile, unsigned long ip_address, unsigned long connect_attempts);
 void log_close(t_host *host);
-#ifdef DEBUG
-void log_error(char *logfile, int code);
-#endif
+void log_error(t_session *session, char *mesg);
 
 #endif
