@@ -8,7 +8,7 @@
 
 #define MAX_EXT 6
 
-char *default_mimetype = "text/plain";
+static char *default_mimetype = "text/plain";
 
 /* Read a mimetype configuration file.
  */
@@ -48,9 +48,13 @@ int read_mimetypes(char *configfile, t_mimetype **mime) {
 		if ((next = strchr(data, ' ')) != NULL) {
 			*next++ = '\0';
 			if (w_mime == NULL) {
-				(*mime)->mimename = w_mime = (t_mimename*)malloc(sizeof(t_mimename));
+				if (((*mime)->mimename = w_mime = (t_mimename*)malloc(sizeof(t_mimename))) == NULL) {
+					return -1;
+				}
 			} else {
-				w_mime->next = (t_mimename*)malloc(sizeof(t_mimename));
+				if ((w_mime->next = (t_mimename*)malloc(sizeof(t_mimename))) == NULL) {
+					return -1;
+				}
 				w_mime = w_mime->next;
 			}
 			w_mime->next = NULL;
@@ -70,9 +74,13 @@ int read_mimetypes(char *configfile, t_mimetype **mime) {
 					}
 					if (strlen(ext) <= MAX_EXT) {
 						if (w_ext == NULL) {
-							(*mime)->extension = w_ext = (t_extension*)malloc(sizeof(t_extension));
+							if (((*mime)->extension = w_ext = (t_extension*)malloc(sizeof(t_extension))) == NULL) {
+								return -1;
+							}
 						} else {
-							w_ext->next = (t_extension*)malloc(sizeof(t_extension));
+							if ((w_ext->next = (t_extension*)malloc(sizeof(t_extension))) == NULL) {
+								return -1;
+							}
 							w_ext = w_ext->next;
 						}
 						w_ext->next = NULL;
