@@ -64,7 +64,8 @@ static int send_chunk(t_request_info *request_info, char *chunk, int size) {
 
 	if (must_send && (chunksize > 0)) {
 		if (request_info->keep_alive) {
-			sprintf(hex, "%x\r\n", chunksize);
+			hex[9] = '\0';
+			snprintf(hex, 9, "%x\r\n", chunksize);
 			result = send_buffer(request_info, hex, strlen(hex));
 		}
 		if (result == 0) {
