@@ -5,7 +5,6 @@
 #include "config.h"
 #include <stdio.h>
 #include <stdlib.h>
-#include <stdbool.h>
 #include <unistd.h>
 #include <time.h>
 #include <string.h>
@@ -13,16 +12,19 @@
 #include <arpa/inet.h>
 #include <pthread.h>
 #include <errno.h>
+#include <fcntl.h>
 #ifdef HAVE_SSL
 #include "libssl.h"
 #endif
-#include "send.h"
-#include "session.h"
 #include "libstr.h"
+#include "libmd5.h"
+#include "send.h"
 
 #define MAX_CHUNK_SIZE 2048
+#define NONCE_DIGITS 10
 
-char *hs_http   = "HTTP/1.1 ";                  //  9
+char *hs_http10 = "HTTP/1.0 ";                  //  9
+char *hs_http11 = "HTTP/1.1 ";                  //  9
 char *hs_server = "Server: ";                   //  8
 char *hs_conn   = "Connection: ";               // 12
 char *hs_concl  = "close\r\n";                  //  7
@@ -80,15 +82,20 @@ static int send_to_client(t_session *session, const char *buffer, int size) {
 			}
 		} else
 #endif
-			bytes_sent = send(session->client_socket, buffer + total_sent, can_send, 0);
+			if ((bytes_sent = send(session->client_socket, (char*)(buffer + total_sent), can_send, 0)) <= 0) {
+				bytes_sent = -1;
+			}
 
 		if (bytes_sent == -1) {
-			close_socket(session);
-			session->keep_alive = false;
-			return -1;
+			if (errno != EINTR) {
+				close_socket(session);
+				session->keep_alive = false;
+				return -1;
+			}
+		} else {
+			total_sent += bytes_sent;
+			session->bytecounter += bytes_sent;
 		}
-		total_sent += bytes_sent;
-		session->bytecounter += bytes_sent;
 	} while (total_sent < size);
 
 	return 0;
@@ -135,7 +142,7 @@ int send_buffer(t_session *session, const char *buffer, int size) {
  */
 int send_header(t_session *session, int code) {
 	int retval = -1;
-	char ecode[4], timestr[64];
+	char ecode[5], timestr[64];
 	const char *emesg;
 	time_t t;
 	struct tm *s;
@@ -146,12 +153,27 @@ int send_header(t_session *session, int code) {
 
 	/* Send HTTP header.
 	 */
-	sprintf(ecode, "%d", code);
+	ecode[4] = '\0';
+	snprintf(ecode, 4, "%d", code);
 	emesg = errorstr(code);
 	do {
-		if (send_buffer(session, hs_http, 9) == -1) {
-			break;
-		} else if (send_buffer(session, ecode, 3) == -1) {
+		if (session->http_version != NULL) {
+			if (*(session->http_version + 7) == '0') {
+				if (send_buffer(session, hs_http10, 9) == -1) {
+					break;
+				}
+			} else {
+				if (send_buffer(session, hs_http11, 9) == -1) {
+					break;
+				}
+			}
+		} else {
+			if (send_buffer(session, hs_http11, 9) == -1) {
+				break;
+			}
+		}
+		
+		if (send_buffer(session, ecode, 3) == -1) {
 			break;
 		} else if (send_buffer(session, " ", 1) == -1) {
 			break;
@@ -208,10 +230,11 @@ int send_header(t_session *session, int code) {
 /* Send a datachunk to the client, used by run_script() in target.c
  */
 static int send_chunk_to_client(t_session *session, const char *chunk, int size) {
-	char hex[8];
+	char hex[10];
 
 	if (session->keep_alive) {
-		if (sprintf(hex, "%x\r\n", size) < 0) {
+		hex[9] = '\0';
+		if (snprintf(hex, 9, "%x\r\n", size) < 0) {
 			return -1;
 		} else if (send_to_client(session, hex, strlen(hex)) == -1) {
 			return -1;
@@ -273,9 +296,13 @@ int send_chunk(t_session *session, const char *chunk, int size) {
  */
 int send_code(t_session *session, int code) {
 	int retval = -1, default_port;
-	char ecode[4], len[8], port[16];
+	char ecode[5], len[10], port[10];
 	const char *emesg;
 
+	if (code == -1) {
+		code = 500;
+	}
+
 	/* Send simple HTTP error message.
 	 */
 	session->mimetype = NULL;
@@ -311,7 +338,8 @@ int send_code(t_session *session, int code) {
 				default_port = 80;
 
 			if (session->binding->port != default_port) {
-				sprintf(port, ":%d", session->binding->port);
+				port[9] = '\0';
+				snprintf(port, 9, ":%d", session->binding->port);
 				if (send_buffer(session, port, strlen(port)) == -1) {
 					break;
 				}
@@ -339,7 +367,11 @@ int send_code(t_session *session, int code) {
 			retval = 0;
 			break;
 		case 401:
-			send_basic_auth(session);
+			if (session->host->auth_method == basic) {
+				send_basic_auth(session);
+			} else {
+				send_digest_auth(session);
+			}
 			retval = 0;
 			break;
 		default:
@@ -350,9 +382,11 @@ int send_code(t_session *session, int code) {
 	if (retval == 0) {
 		do {
 			retval = -1;
-			sprintf(ecode, "%d", code);
+			ecode[4] = '\0';
+			snprintf(ecode, 4, "%d", code);
 			emesg = errorstr(code);
-			sprintf(len, "%d", 2 * strlen(emesg) + 291);
+			len[9] = '\0';
+			snprintf(len, 9, "%d", 2 * strlen(emesg) + 291);
 
 			if (send_buffer(session, "Content-Length: ", 16) == -1) {
 				break;
@@ -400,28 +434,153 @@ int send_code(t_session *session, int code) {
 	return retval;
 }
 
-/* Send a Basic Authentication message to the client.
+/* Send directly to buffer, unbuffered
  */
-int send_basic_auth(t_session *session) {
-	int retval = -1;
+int send_directly(int sock, const char *buffer, int size) {
+	int total_sent = 0, bytes_sent;
 
-	do {
-		if (send_buffer(session, "WWW-Authenticate: Basic", 23) == -1) {
-			break;
-		} else if (session->host->login_message != NULL) {
-			if (send_buffer(session, " realm=\"", 8) == -1) {
-				break;
-			} else if (send_buffer(session, session->host->login_message, strlen(session->host->login_message)) == -1) {
-				break;
-			} else if (send_buffer(session, "\"", 1) == -1) {
-				break;
+	if (size <= 0) {
+		return 0;
+	} else while (total_sent < size) {
+		if ((bytes_sent = send(sock, buffer + total_sent, size - total_sent, 0)) == -1) {
+			if (errno != EINTR) {
+				return -1;
+			}
+		} else {
+			total_sent += bytes_sent;
+		}
+	}
+
+	return 0;
+}
+
+static int set_padding(t_fcgi_buffer *fcgi_buffer, bool adjust_buffer) {
+	unsigned char padding;
+
+	if ((padding = (fcgi_buffer->data[5] & 7)) > 0) {
+		padding = 8 - padding;
+		if (adjust_buffer) {
+			bzero(fcgi_buffer->data + fcgi_buffer->size, (int)padding);
+			fcgi_buffer->size += (int)padding;
+		}
+	}
+	fcgi_buffer->data[6] = padding;
+
+	return (int)padding;
+}
+
+int send_fcgi_buffer(t_fcgi_buffer *fcgi_buffer, const char *buffer, int size) {
+	int padding;
+
+	if (size > FCGI_BUFFER_SIZE) {
+		if (fcgi_buffer->size > 0) {
+			set_padding(fcgi_buffer, true);
+			if (send_directly(fcgi_buffer->sock, (char*)fcgi_buffer->data, fcgi_buffer->size) == -1) {
+				return -1;
 			}
 		}
-		if (send_buffer(session, "\r\n", 2) == -1) {
-			break;
-		}
-		retval = 0;
-	} while (false);
 
-	return retval;
+		memcpy(fcgi_buffer->data, "\x01\x00\x00\x01" "\xff\xff\x00\x00", 8);
+		fcgi_buffer->data[1] = fcgi_buffer->mode;
+		fcgi_buffer->data[4] = (FCGI_BUFFER_SIZE >> 8 ) & 255;
+		fcgi_buffer->data[5] = FCGI_BUFFER_SIZE & 255;
+
+		padding = set_padding(fcgi_buffer, false);
+		if (send_directly(fcgi_buffer->sock, (char*)fcgi_buffer->data, 8) == -1) {
+			return -1;
+		} else if (send_directly(fcgi_buffer->sock, buffer, size) == -1) {
+			return -1;
+		}
+
+		fcgi_buffer->size = 0;
+		bzero(fcgi_buffer->data, padding);
+
+		if (send_directly(fcgi_buffer->sock, (char*)fcgi_buffer->data, padding) == -1) {
+			return -1;
+		} else if (send_fcgi_buffer(fcgi_buffer, buffer + FCGI_BUFFER_SIZE, size - FCGI_BUFFER_SIZE) == -1) {
+			return -1;
+		}
+	} else if (buffer == NULL) {
+		if (fcgi_buffer->size > 0) {
+			set_padding(fcgi_buffer, true);
+			if (send_directly(fcgi_buffer->sock, (char*)fcgi_buffer->data, fcgi_buffer->size) == -1) {
+				return -1;
+			}
+		}
+
+		memcpy(fcgi_buffer->data, "\x01\x00\x00\x01" "\x00\x00\x00\x00", 8);
+		fcgi_buffer->data[1] = fcgi_buffer->mode;
+		set_padding(fcgi_buffer, true);
+		if (send_directly(fcgi_buffer->sock, (char*)fcgi_buffer->data, 8) == -1) {
+			return -1;
+		}
+
+		fcgi_buffer->size = 0;
+	} else {
+		if ((fcgi_buffer->size + size > FCGI_BUFFER_SIZE) && (fcgi_buffer->size > 0)) {
+			set_padding(fcgi_buffer, true);
+			if (send_directly(fcgi_buffer->sock, (char*)fcgi_buffer->data, fcgi_buffer->size) == -1) {
+				return -1;
+			}
+			fcgi_buffer->size = 0;
+		}
+
+		if (fcgi_buffer->size == 0) {
+			memcpy(fcgi_buffer->data, "\x01\x00\x00\x01" "\x00\x00\x00\x00", 8);
+			fcgi_buffer->data[1] = fcgi_buffer->mode;
+			fcgi_buffer->size = 8;
+		}
+		memcpy(fcgi_buffer->data + fcgi_buffer->size, buffer, size);
+		fcgi_buffer->size += size;
+		fcgi_buffer->data[4] = ((fcgi_buffer->size - 8) >> 8) & 255;
+		fcgi_buffer->data[5] = (fcgi_buffer->size - 8) & 255;
+	}
+
+	return 0;
+}
+
+/* Send a Basic Authentication message to the client.
+ */
+void send_basic_auth(t_session *session) {
+	if (send_buffer(session, "WWW-Authenticate: Basic", 23) == -1) {
+		return;
+	} else if (session->host->login_message != NULL) {
+		if (send_buffer(session, " realm=\"", 8) == -1) {
+			return;
+		} else if (send_buffer(session, session->host->login_message, strlen(session->host->login_message)) == -1) {
+			return;
+		} else if (send_buffer(session, "\"", 1) == -1) {
+			return;
+		}
+	}
+	send_buffer(session, "\r\n", 2);
+}
+
+/* Send a Digest Authentication message to the client.
+ */
+void send_digest_auth(t_session *session) {
+	char nonce[2 * NONCE_DIGITS];
+	int i;
+
+	for (i = 0; i < NONCE_DIGITS; i++) {
+		sprintf(nonce + (2 * i), "%02X", (char)(random() & 255));
+	}
+	
+	if (send_buffer(session, "WWW-Authenticate: Digest", 24) == -1) {
+		return;
+	} else if (session->host->login_message != NULL) {
+		if (send_buffer(session, " realm=\"", 8) == -1) {
+			return;
+		} else if (send_buffer(session, session->host->login_message, strlen(session->host->login_message)) == -1) {
+			return;
+		} else if (send_buffer(session, "\"", 1) == -1) {
+			return;
+		}
+	}
+	if (send_buffer(session, ", nonce=\"", 9) == -1) {
+		return;
+	} else if (send_buffer(session, nonce, 2 * NONCE_DIGITS) == -1) {
+		return;
+	}
+	send_buffer(session, "\", algorithm=MD5, stale=false\r\n", 31);
 }
