@@ -11,6 +11,7 @@
 #endif
 
 typedef enum { root, part } t_pathmatch;
+typedef enum { basic, digest } t_auth_method;
 
 typedef struct type_cgi_handler {
 	char          *handler;
@@ -18,6 +19,34 @@ typedef struct type_cgi_handler {
 	struct type_cgi_handler *next;
 } t_cgi_handler;
 
+typedef struct type_connect_to {
+	int           port;
+	char          *host;
+	bool          available;
+
+	struct type_connect_to *next;
+} t_connect_to;
+
+typedef struct type_cgi_session {
+	unsigned long client_ip;
+	t_connect_to  *connect_to;
+	int timer;
+
+	struct type_cgi_session *next;
+} t_cgi_session;
+
+typedef struct type_fcgi_server {
+	char          *fcgi_id;
+	t_connect_to  *connect_to;
+	t_charlist    extension;
+	int           session_timeout;
+
+	t_cgi_session *cgi_session_list[256];
+	pthread_mutex_t cgi_session_mutex[256];
+
+	struct type_fcgi_server *next;
+} t_fcgi_server;
+
 typedef struct type_throttle {
 	char          *filetype;
 	unsigned long upload_speed;
@@ -28,7 +57,7 @@ typedef struct type_throttle {
 typedef struct type_binding {
 	int           port;
 	unsigned long interface;
-	char          *name;
+	char          *binding_id;
 #ifdef HAVE_SSL
 	bool          use_ssl;
 	char          *server_key;
@@ -51,6 +80,7 @@ typedef struct type_directory {
 	t_pathmatch   path_match;
 	char          *cgi_wrap_id;
 	t_groups      groups;
+	char          *start_file;
 	bool          execute_cgi;
 	bool          execute_cgiset;
 	bool          show_index;
@@ -58,6 +88,7 @@ typedef struct type_directory {
 	bool          follow_symlinks;
 	bool          follow_symlinksSet;
 	char          *passwordfile;
+	t_auth_method auth_method;
 	char          *groupfile;
 	t_charlist    required_group;
 	t_accesslist  *access_list;
@@ -94,6 +125,7 @@ typedef struct type_host {
 #endif
 	char          *login_message;
 	char          *passwordfile;
+	t_auth_method auth_method;
 	char          *groupfile;
 	char          *index_style;
 	t_charlist    required_binding;
@@ -114,6 +146,7 @@ typedef struct type_host {
 	bool          prevent_sqli;
 	bool          prevent_xss;
 	bool          follow_symlinks;
+	t_charlist    fast_cgi;
 
 	struct type_host *next;
 } t_host;
@@ -137,6 +170,7 @@ typedef struct type_config {
 	int           time_for_cgi;
 	char          *system_logfile;
 	char          *garbage_logfile;
+	char          *pidfile;
 	t_accesslist  *logfile_mask;
 	
 #ifdef HAVE_PLUGIN
@@ -164,6 +198,13 @@ typedef struct type_config {
 	int           flooding_time;
 	int           reconnect_delay;
 	t_accesslist  *banlist_mask;
+	t_fcgi_server *fcgi_server;
+
+#ifdef HAVE_CACHE
+	int           cache_size;
+	int           cache_max_filesize;
+	int           cache_min_filesize;
+#endif
 
 #ifdef HAVE_COMMAND
 	t_binding     *command_port;
