@@ -13,10 +13,6 @@
 #include <pwd.h>
 #include "libstr.h"
 #include "session.h"
-#include "liblist.h"
-#ifdef HAVE_SSL
-#include "libssl.h"
-#endif
 
 int new_client_id = 0;
 
@@ -38,6 +34,7 @@ static void clear_session(t_session *session) {
 	session->head_request = false;
 	session->is_cgi_script = false;
 	session->cgi_handler = NULL;
+	session->fcgi_server = NULL;
 	session->method = NULL;
 	session->uri = NULL;
 	session->uri_is_dir = false;
@@ -87,8 +84,6 @@ void init_session(t_session *session) {
 	}
 	session->force_quit = false;
 	session->kept_alive = 0;
-	session->flooding_counter = 0;
-	session->flooding_timer = session->config->flooding_time;
 #ifdef HAVE_SSL
 	session->ssl_data = NULL;
 #endif
@@ -97,6 +92,8 @@ void init_session(t_session *session) {
 	session->buffer_size = 0;
 	session->bytes_in_buffer = 0;
 
+	session->flooding_timer = time(NULL);
+
 	clear_session(session);
 }
 
@@ -108,7 +105,6 @@ void reset_session(t_session *session) {
 	sfree(session->local_user);
 	sfree(session->remote_user);
 	sfree(session->file_on_disk);
-	//sfree(session->chunk_buffer);
 	remove_headerfields(session);
 	if (session->directory != NULL) {
 		pthread_mutex_lock(&(session->directory->client_mutex));
@@ -146,9 +142,11 @@ void reset_session(t_session *session) {
 	}
 	
 	clear_session(session);
-	session->flooding_counter++;
 }
 
+//void destroy_session(t_session *session) {
+//}
+
 /* Parse the HTTP headerfields from the received request.
  */
 t_headerfield *parse_headerfields(char *line) {
@@ -205,7 +203,7 @@ char *get_headerfield(char *key, t_headerfield *headerfields) {
 
 	len = strlen(key);
 	while (headerfields != NULL) {
-		if (str_ncompare(headerfields->data, key, len) == 0) {
+		if (strncasecmp(headerfields->data, key, len) == 0) {
 			retval = headerfields->data + headerfields->value_offset;
 			break;
 		}
@@ -234,9 +232,11 @@ int get_homedir(t_session *session, char *username) {
 	struct passwd *pwd;
 
 	if ((pwd = getpwnam(username)) != NULL) {
-		session->host->website_root = (char*)malloc(strlen(pwd->pw_dir) + 13);
-		strcpy(session->host->website_root, pwd->pw_dir);
-		strcat(session->host->website_root, "/public_html");
+		if ((session->host->website_root = (char*)malloc(strlen(pwd->pw_dir) + 13)) == NULL) {
+			return -1;
+		}
+
+		sprintf(session->host->website_root, "%s%s", pwd->pw_dir, "/public_html");
 		session->websiteroot_copied = true;
 		return 200;
 	}
@@ -329,6 +329,9 @@ int copy_directory_settings(t_session *session) {
 					if (dir->cgi_wrap_id != NULL) {
 						session->host->cgi_wrap_id = dir->cgi_wrap_id;
 					}
+					if (dir->start_file != NULL) {
+						session->host->start_file = dir->start_file;
+					}
 					if (dir->execute_cgiset) {
 						session->host->execute_cgi = dir->execute_cgi;
 					}
@@ -347,11 +350,12 @@ int copy_directory_settings(t_session *session) {
 						session->host->imgref_replacement = dir->imgref_replacement;
 					}
 					if (dir->passwordfile != NULL) {
+						session->host->auth_method = dir->auth_method;
 						session->host->passwordfile = dir->passwordfile;
 					}
 					if (dir->groupfile != NULL) {
-						session->host->groupfile = dir->groupfile;
 						session->host->required_group = dir->required_group;
+						session->host->groupfile = dir->groupfile;
 					}
 					break;
 				} else {
@@ -365,127 +369,6 @@ int copy_directory_settings(t_session *session) {
 	return 200;
 }
 
-/* Wrapper for setenv
- */
-static int add_to_environment(char *key, char *value, char *buffer, int pos, int max) {
-	int key_len, value_len;
-
-	if (buffer == NULL) {
-		setenv(key, value, 1);
-
-		return 0;
-	} else {
-		key_len = strlen(key);
-		value_len = strlen(value);
-
-		if (pos + 2 + key_len + value_len <= max) {
-			buffer += pos;
-			*buffer = (char)key_len;
-			*(buffer + 1) = (char)value_len;
-			memcpy(buffer + 2, key, key_len);
-			memcpy(buffer + 2 + key_len, value, value_len);
-
-			return 2 + key_len + value_len;
-		} else {
-			return 0;
-		}
-	}
-}
-
-/* Copy a headerfield to an environment setting
- */
-static int headerfield_to_environment(t_session *session, char *key, char *envir, char *buffer, int pos, int max) {
-	char *value;
-
-	if ((value = get_headerfield(key, session->headerfields)) != NULL) {
-		return add_to_environment(envir, value, buffer, pos, max);
-	} else {
-		return 0;
-	}
-}
-
-/* Set environment variables for CGI script.
- */
-int set_environment(t_session *session, char *buffer, int max) {
-	char ip[16], len[16], value[16];
-	unsigned char *ip_address;
-	t_keyvalue *envir;
-	int pos = 0;
-
-	pos += add_to_environment("GATEWAY_INTERFACE", "CGI/1.1", buffer, pos, max);
-	pos += add_to_environment("DOCUMENT_ROOT", session->host->website_root, buffer, pos, max);
-	pos += add_to_environment("REQUEST_METHOD", session->method, buffer, pos, max);
-	pos += add_to_environment("REQUEST_URI", session->uri, buffer, pos, max);
-	pos += add_to_environment("SCRIPT_NAME", session->uri, buffer, pos, max);
-	pos += add_to_environment("SCRIPT_FILENAME", session->file_on_disk, buffer, pos, max);
-
-	ip_address = (unsigned char*)&(session->ip_address);
-	sprintf(ip, "%hhu.%hhu.%hhu.%hhu", ip_address[0], ip_address[1], ip_address[2], ip_address[3]);
-	pos += add_to_environment("REMOTE_ADDR", ip, buffer, pos, max);
-
-	if (session->vars != NULL) {
-		pos += add_to_environment("QUERY_STRING", session->vars, buffer, pos, max);
-	}
-
-	if (session->body != NULL) {
-		sprintf(len, "%ld", session->content_length);
-		pos += add_to_environment("CONTENT_LENGTH", len, buffer, pos, max);
-		pos + headerfield_to_environment(session, "Content-Type:", "CONTENT_TYPE", buffer, pos, max);
-	}
-
-	sprintf(value, "%d", session->binding->port);
-	pos += add_to_environment("SERVER_PORT", value, buffer, pos, max);
-	pos += add_to_environment("SERVER_NAME", *(session->host->hostname.item), buffer, pos, max);
-	pos += add_to_environment("SERVER_PROTOCOL", session->http_version, buffer, pos, max);
-	pos += add_to_environment("SERVER_SOFTWARE", "Hiawatha v"VERSION, buffer, pos, max);
-	pos += add_to_environment("REDIRECT_STATUS", "CGI", buffer, pos, max);
-
-	if (session->remote_user != NULL) {
-		pos += add_to_environment("AUTH_TYPE", "Basic", buffer, pos, max);
-		pos += add_to_environment("REMOTE_USER", session->remote_user, buffer, pos, max);
-	}
-
-#ifdef HAVE_SSL
-	if (session->binding->use_ssl) {
-		pos += add_to_environment("CONNECTION_TYPE", "SSL", buffer, pos, max);
-	} else
-#endif
-		pos += add_to_environment("CONNECTION_TYPE", "plain", buffer, pos, max);
-
-	pos += headerfield_to_environment(session, "Accept:", "HTTP_ACCEPT", buffer, pos, max);
-	pos += headerfield_to_environment(session, "Accept-Charset:", "HTTP_ACCEPT_CHARSET", buffer, pos, max);
-	pos += headerfield_to_environment(session, "Accept-Language:", "HTTP_ACCEPT_LANGUAGE", buffer, pos, max);
-	pos += headerfield_to_environment(session, "Client-IP:", "HTTP_CLIENT_IP", buffer, pos, max);
-	pos += headerfield_to_environment(session, "From:", "HTTP_FROM", buffer, pos, max);
-	pos += headerfield_to_environment(session, "Host:", "HTTP_HOST", buffer, pos, max);
-	pos += headerfield_to_environment(session, "Range:", "HTTP_RANGE", buffer, pos, max);
-	pos += headerfield_to_environment(session, "Referer:", "HTTP_REFERER", buffer, pos, max);
-	pos += headerfield_to_environment(session, "User-Agent:", "HTTP_USER_AGENT", buffer, pos, max);
-	pos += headerfield_to_environment(session, "Via:", "HTTP_VIA", buffer, pos, max);
-	pos += headerfield_to_environment(session, "X-Forwarded-For:", "HTTP_X_FORWARDED_FOR", buffer, pos, max);
-
-	sprintf(value, "%d", session->return_code);
-	pos += add_to_environment("HTTP_RETURN_CODE", value, buffer, pos, max);
-	if (session->error_code != -1) {
-		sprintf(value, "%d", session->error_code);
-		pos += add_to_environment("HTTP_GENERATED_ERROR", value, buffer, pos, max);
-	}
-
-	if (session->cookie != NULL) {
-		pos += add_to_environment("HTTP_COOKIE", session->cookie, buffer, pos, max);
-	} else {
-		pos += headerfield_to_environment(session, "Cookie:", "HTTP_COOKIE", buffer, pos, max);
-	}
-
-	envir = session->host->envir_str;
-	while (envir != NULL) {
-		pos += add_to_environment(envir->key, envir->value, buffer, pos, max);
-		envir = envir->next;
-	}
-
-	return pos;
-}
-
 /* Complete path in chrooted session?
  *  *  */
 char *file_in_chroot(t_session *session, char *file) {
