@@ -1,11 +1,12 @@
 #ifndef _SESSION_H
 #define _SESSION_H
 
+#include <time.h>
 #ifdef HAVE_SSL
 #include "libssl.h"
 #endif
-#include "serverconfig.h"
 #include "liblist.h"
+#include "serverconfig.h"
 
 #define TIMER_OFF -1
 #define MAX_CHILD_ID 30000
@@ -40,6 +41,7 @@ typedef struct type_session {
 	int            kept_alive;
 	bool           is_cgi_script;
 	char           *cgi_handler;
+	t_fcgi_server  *fcgi_server;
 	char           *request, *method, *uri, *vars, *http_version, *body, *file_on_disk;
 	long           header_length, content_length, buffer_size, bytes_in_buffer;
 	char           *local_user;
@@ -74,8 +76,7 @@ typedef struct type_session {
 
 	/* Flooding protection
 	 */
-	int            flooding_timer;
-	int            flooding_counter;
+	time_t         flooding_timer;
 
 	/* SSL
 	 */
@@ -104,7 +105,6 @@ bool duplicate_host(t_session *session);
 void restore_host(t_session *session);
 bool is_volatile_object(t_session *session);
 int  copy_directory_settings(t_session *session);
-int  set_environment(t_session *session, char *buffer, int max);
 char *file_in_chroot(t_session *session, char *file);
 bool client_is_rejected_bot(t_session *session);
 void close_socket(t_session *session);
