@@ -19,12 +19,18 @@
 #ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
 #endif
-#include "target.h"
-#include "log.h"
+#ifdef HAVE_SENDFILE
+#include <sys/sendfile.h>
+#endif
 #include "libstr.h"
 #include "libfs.h"
-#include "libcgi.h"
+#include "target.h"
+#include "log.h"
+#include "cgi.h"
 #include "send.h"
+#ifdef HAVE_CACHE
+#include "cache.h"
+#endif
 #ifdef HAVE_PLUGIN
 #include "plugin.h"
 #endif
@@ -34,6 +40,9 @@
 #define MAX_CGI_HEADER          8192
 #define MAX_TRACE_HEADER        1024
 
+#define FILESIZE_BUFFER_SIZE      30
+#define CGI_BUFFER_SIZE        65790  // fcgi.data + fcgi.padding
+
 #define rs_QUIT       -1
 #define rs_DISCONNECT -2
 #define rs_FORCE_QUIT -3
@@ -55,11 +64,15 @@ extern unsigned long req_files, req_cgi, req_index;
  */
 int send_file(t_session *session) {
 	char *referer, *buffer = NULL, value[64], *dot, *date, *range, *range_start, *range_end;
-	long bytes_read, total_bytes, size, file_size, send_start, send_end, send_size, speed;
+	long bytes_read, total_bytes, size, speed;
+	off_t file_size, send_start, send_end, send_size;
 	int  retval, handle, http_code;
 	bool invalid_referer = false, prot_oke;
 	struct stat status;
 	struct tm *fdate;
+#ifdef HAVE_CACHE
+	t_cached_object *cached_object;
+#endif
 
 #ifdef HAVE_COMMAND
 	req_files++;
@@ -118,7 +131,7 @@ int send_file(t_session *session) {
 	handle = -1;
 	if (session->accept_gzip) {
 		size = strlen(session->file_on_disk);
-		strcpy(session->file_on_disk + size, ".gz");
+		memcpy(session->file_on_disk + size, ".gz\0", 4);
 		if ((handle = open(session->file_on_disk, O_RDONLY)) != -1) {
 			session->encode_gzip = true;
 		} else {
@@ -140,6 +153,7 @@ int send_file(t_session *session) {
 		switch (contains_not_allowed_symlink(session->file_on_disk, session->host->website_root)) {
 			case error:
 				close(handle);
+				log_error(session, "target,send_file,symlinks");
 				return 500;
 			case not_found:
 				close(handle);
@@ -191,6 +205,7 @@ int send_file(t_session *session) {
 
 	if ((file_size = filesize(session->file_on_disk)) == -1) {
 		close(handle);
+		log_error(session, "target,send_file,filesize");
 		return 500;
 	}
 	send_start = 0;
@@ -199,52 +214,54 @@ int send_file(t_session *session) {
 
 	/* Range
 	 */
-	if ((range = get_headerfield("Range:", session->headerfields)) != NULL) {
-		if (memcmp(range, "bytes=", 6) == 0) {
-			range += 6;
-			if (split_string(range, &range_start, &range_end, '-') == 0) {
+	if (*(session->http_version + 7) == '1') {
+		if ((range = get_headerfield("Range:", session->headerfields)) != NULL) {
+			if (memcmp(range, "bytes=", 6) == 0) {
+				range += 6;
+				if (split_string(range, &range_start, &range_end, '-') == 0) {
 
-				if (*range_start != '\0') {
-					if ((send_start = str2int(range_start)) >= 0) {
-						if (*range_end != '\0') {
-							if ((send_end = str2int(range_end)) >= 0) {
-								// bytes=XX-XX
+					if (*range_start != '\0') {
+						if ((send_start = str2int(range_start)) >= 0) {
+							if (*range_end != '\0') {
+								if ((send_end = str2int(range_end)) >= 0) {
+									// bytes=XX-XX
+									http_code = 206;
+								}
+							} else {
+								// bytes=XX-
 								http_code = 206;
 							}
-						} else {
-							// bytes=XX-
+						}
+					} else {
+						if ((send_start = str2int(range_end)) >= 0) {
+							// bytes=-XX
+							send_start = file_size - send_start;
 							http_code = 206;
 						}
 					}
-				} else {
-					if ((send_start = str2int(range_end)) >= 0) {
-						// bytes=-XX
-						send_start = file_size - send_start;
-						http_code = 206;
-					}
-				}
 
-				if (http_code == 206) {
-					if (send_start >= file_size) {
-						close(handle);
-						return 416;
-					}
-					if (send_start < 0) {
-						send_start = 0;
-					}
-					if (send_end >= file_size) {
-						send_end = file_size - 1;
-					}
-					if (send_start <= send_end) {
-						send_size = send_end - send_start + 1;
+					if (http_code == 206) {
+						if (send_start >= file_size) {
+							close(handle);
+							return 416;
+						}
+						if (send_start < 0) {
+							send_start = 0;
+						}
+						if (send_end >= file_size) {
+							send_end = file_size - 1;
+						}
+						if (send_start <= send_end) {
+							send_size = send_end - send_start + 1;
+						} else {
+							close(handle);
+							return 416;
+						}
 					} else {
-						close(handle);
-						return 416;
+						send_start = 0;
+						send_end = file_size - 1;
+						send_size = file_size;
 					}
-				} else {
-					send_start = 0;
-					send_end = file_size - 1;
-					send_size = file_size;
 				}
 			}
 		}
@@ -272,10 +289,11 @@ int send_file(t_session *session) {
 
 		/* Content-Range
 		 */
+		value[63] = '\0';
 		if (http_code == 206) {
 			if (send_buffer(session, "Content-Range: bytes ", 21) == -1) {
 				break;
-			} else if (sprintf(value, "%ld-%ld/%ld\r\n", send_start, send_end, file_size) == -1) {
+			} else if (snprintf(value, 63, "%lld-%lld/%lld\r\n", (long long)send_start, (long long)send_end, file_size) == -1) {
 				break;
 			} else if (send_buffer(session, value, strlen(value)) == -1) {
 				break;
@@ -284,7 +302,7 @@ int send_file(t_session *session) {
 
 		if (send_buffer(session, hs_conlen, 16) == -1) {
 			break;
-		} else if (sprintf(value, "%ld\r\n\r\n", send_size) == -1) {
+		} else if (snprintf(value, 63, "%lld\r\n\r\n", (long long)send_size) == -1) {
 			break;
 		} else if (send_buffer(session, value, strlen(value)) == -1) {
 			break;
@@ -323,13 +341,33 @@ int send_file(t_session *session) {
 				/* Normal file
 				 */
 #ifdef HAVE_SENDFILE
+				/* This is for sendfile() under Linux. It turned out that it's
+				 * slower (about 5%) then Hiawatha's 'sendfile' code.
+				 */
 				off_t offset;
 
-				if (session->throttle == 0) {
+				if ((session->throttle == 0) 
+#ifdef HAVE_SSL
+						&& (session->binding->use_ssl == false)
+#endif
+						) {
+					send_buffer(session, NULL, 0);
 					offset = send_start;
-					sendfile(session->client_socket, handle, &offset, send_size);
+					if (sendfile(session->client_socket, handle, &offset, send_size) == -1) {
+						retval = -1;
+					}
 				} else 
 #endif
+
+#ifdef HAVE_CACHE
+				if ((cached_object = search_cache(session, session->file_on_disk)) != NULL) {
+					send_buffer(session, cached_object->data, cached_object->size);
+					done_with_cached_object(cached_object);
+				} else if ((cached_object = add_to_cache(session, session->file_on_disk)) != NULL) {
+					send_buffer(session, cached_object->data, cached_object->size);
+					done_with_cached_object(cached_object);
+				} else
+#endif
 				if ((buffer = (char*)malloc(FILE_BUFFER_SIZE)) != NULL) {
 					while ((send_size > 0) && (retval == 200)) {
 						switch ((bytes_read = read(handle, buffer, FILE_BUFFER_SIZE))) {
@@ -367,27 +405,31 @@ int send_file(t_session *session) {
 
 /* Run a CGI program and send output to the client.
  */
-int run_cgi_program(t_session *session) {
-	int retval = 200, result, handle, CGI_pid;
-	char *line, *new_line, *contyp_start, *contyp_end;
-	bool in_body = false, send_in_chunks = true, wrap_CGI;
+int run_script(t_session *session) {
+	int retval = 200, result, handle, cgi_pid = -1;
+	char *line, *new_line, *contyp_start, *contyp_end, *code;
+	bool in_body = false, send_in_chunks = true, wrap_cgi, use_fast_cgi;
+	t_connect_to *connect_to;
 	t_cgi_info cgi_info;
 
 #ifdef HAVE_COMMAND
 	req_cgi++;
 #endif
 
-	wrap_CGI = (session->config->cgi_wrapper != NULL) && ((session->host->cgi_wrap_id != NULL) ||
-		((session->local_user != NULL) && session->config->wrap_user_cgi));
+	if ((use_fast_cgi = (session->fcgi_server != NULL))) {
+		wrap_cgi = false;
+	} else {
+		wrap_cgi = (session->config->cgi_wrapper != NULL) && ((session->host->cgi_wrap_id != NULL) ||
+			((session->local_user != NULL) && session->config->wrap_user_cgi));
+	}
 
-
-	/* HTTP/1.0 has no knowledge about chunked Transfer-Encoding.
+	/* HTTP/1.0 does not support chunked Transfer-Encoding.
 	 */
 	if (*(session->http_version + 7) == '0') {
 		session->keep_alive = false;
 	}
 
-	if (wrap_CGI == false) {
+	if ((wrap_cgi == false) && (use_fast_cgi == false)) {
 		if ((handle = open(session->file_on_disk, O_RDONLY)) == -1) {
 			if (errno == EACCES) {
 				set_403_reason(session, fb_FILESYSTEM);
@@ -404,10 +446,11 @@ int run_cgi_program(t_session *session) {
 		return 403;
 	}
 
-	if (wrap_CGI == false) {
+	if ((wrap_cgi == false) && (use_fast_cgi == false)) {
 		if (session->cgi_handler == NULL) {
 			switch (can_execute(session->file_on_disk, session->config->server_uid, session->config->server_gid, &(session->config->groups))) {
 				case error:
+					log_error(session, "target,run_script,can_execute");
 					return 500;
 				case not_found:
 					return 404;
@@ -423,8 +466,10 @@ int run_cgi_program(t_session *session) {
 		if (session->host->follow_symlinks == false) {
 			switch (contains_not_allowed_symlink(session->file_on_disk, session->host->website_root)) {
 				case error:
+					log_error(session, "target,run_script,symlink");
 					return 500;
 				case not_found:
+printf("404-2\n");
 					return 404;
 				case no_access:
 				case yes:
@@ -448,6 +493,7 @@ int run_cgi_program(t_session *session) {
 		if (session->vars != NULL) {
 			switch (prevent_SQLi(session->vars, strlen(session->vars) + 1, &line)) {
 				case -1:
+					log_error(session, "target,run_script,prevent_SQLi,URL");
 					return 500;
 				case 0:
 					break;
@@ -465,6 +511,7 @@ int run_cgi_program(t_session *session) {
 			session->content_length = parse_special_chars(session->body, session->content_length);
 			switch (result = prevent_SQLi(session->body, session->content_length, &line)) {
 				case -1:
+					log_error(session, "target,run_script,prevent_SQLi,postdata");
 					return 500;
 				case 0:
 					break;
@@ -483,6 +530,7 @@ int run_cgi_program(t_session *session) {
 			parse_special_chars(session->cookie, strlen(session->cookie));
 			switch (prevent_SQLi(session->cookie, strlen(session->cookie), &line)) {
 				case -1:
+					log_error(session, "target,run_script,preven_SQLi,cookie");
 					return 500;
 				case 0:
 					break;
@@ -513,10 +561,27 @@ int run_cgi_program(t_session *session) {
 
 	cgi_info.input_buffer_size = cgi_info.error_buffer_size = CGI_BUFFER_SIZE;
 	cgi_info.input_len = cgi_info.error_len = 0;
-	cgi_info.wrap_cgi = wrap_CGI;
+	cgi_info.wrap_cgi = wrap_cgi;
 
-	if ((CGI_pid = fork_CGI_process(session, &cgi_info)) == -1) {
-		return 500;
+	if (use_fast_cgi) {
+		if ((connect_to = select_connect_to(session->fcgi_server, session->ip_address)) == NULL) {
+			return 503;
+		} else if ((cgi_info.from_cgi = connect_to_fcgi_server(connect_to)) == -1) {
+			connect_to->available = false;
+			log_string(session->config->system_logfile, "Can't connect to FastCGI server");
+			return 503;
+		} else {
+			connect_to->available = true;
+			if (send_fcgi_data(session, cgi_info.from_cgi) == -1) {
+				log_error(session, "target,run_script,send_fcgi_data");
+				return 500;
+			}
+		}
+	} else {
+		if ((cgi_pid = fork_cgi_process(session, &cgi_info)) == -1) {
+			log_error(session, "target,run_script,fork_cgi_process");
+			return 500;
+		}
 	}
 
 	if ((cgi_info.input_buffer = (char*)malloc(cgi_info.input_buffer_size + 1)) == NULL) {
@@ -527,9 +592,13 @@ int run_cgi_program(t_session *session) {
 	}
 
 	if (retval != 200) {
-		close(cgi_info.to_cgi);
-		close(cgi_info.from_cgi);
-		close(cgi_info.cgi_error);
+		if (use_fast_cgi) {
+			close(cgi_info.from_cgi);
+		} else {
+			close(cgi_info.to_cgi);
+			close(cgi_info.from_cgi);
+			close(cgi_info.cgi_error);
+		}
 		return retval;
 	}
 
@@ -538,29 +607,34 @@ int run_cgi_program(t_session *session) {
 	cgi_info.timer = session->config->time_for_cgi;
 
 	do {
-		result = read_from_CGI_process(session, &cgi_info);
+		if (use_fast_cgi) {
+			result = read_from_fcgi_server(session, &cgi_info);
+		} else {
+			result = read_from_cgi_process(session, &cgi_info);
+		}
 
 		switch (result) {
-			case rc_SELECT_ERROR:
-			case rc_TIMEOUT:
+			case cr_ERROR:
+			case cr_TIMEOUT:
 				if (in_body) {
 					retval = rs_DISCONNECT;
 				} else {
+					log_error(session, "target,run_script,in_body");
 					retval = 500;
 				}
 				break;
-			case rc_FORCE_QUIT:
+			case cr_FORCE_QUIT:
 				retval = rs_FORCE_QUIT;
 				break;
-			case rc_OKE:
+			case cr_OKE:
 				switch (cgi_info.input_result) {
-					case rc_READ_DATA:
+					case cd_READ_DATA:
 						/* Data received from CGI
 						 */
 						if (in_body) {
+							/* Read content
+							 */
 							if (session->head_request == false) {
-								/* Read content
-								 */
 								if (send_in_chunks) {
 									if (send_chunk(session, cgi_info.input_buffer, cgi_info.input_len) == -1) {
 										retval = rs_DISCONNECT;
@@ -579,7 +653,7 @@ int run_cgi_program(t_session *session) {
 
 							if ((new_line = strstr(cgi_info.input_buffer, "\r\n\r\n")) != NULL) {
 								if (session->throttle == 0) {
-									if ((contyp_start = str_search(cgi_info.input_buffer, hs_contyp)) != NULL) {
+									if ((contyp_start = strcasestr(cgi_info.input_buffer, hs_contyp)) != NULL) {
 										if ((contyp_end = strchr(contyp_start, '\r')) != NULL) {
 											contyp_start += 14;
 											*contyp_end = '\0';
@@ -589,19 +663,39 @@ int run_cgi_program(t_session *session) {
 									}
 								}
 
-								if (str_search(cgi_info.input_buffer, "Location:")) {
+								if (strcasestr(cgi_info.input_buffer, "Location:") != NULL) {
 									if (send_header(session, 302) == -1) {
 										retval = rs_DISCONNECT;
 										break;
 									}
+								} else if ((code = strcasestr(cgi_info.input_buffer, "Status: ")) != NULL) {
+									line = code += 8;
+									while (*line != '\0') {
+										if (*line == '\r') {
+											*line = '\0';
+										}
+										line++;
+									}
+									if ((retval = atoi(code)) == 0) {
+										log_error(session, "target,run_script,statuscode");
+										retval = 500;
+									}
+									if (send_header(session, retval) == -1) {
+										retval = rs_DISCONNECT;
+									}
+									break;
 								} else if (send_header(session, session->return_code) == -1) {
 									retval = rs_DISCONNECT;
 									break;
 								}
 								if (session->return_code == 401) {
-									send_basic_auth(session);
+									if (session->host->auth_method == basic) {
+										send_basic_auth(session);
+									} else {
+										send_digest_auth(session);
+									}
 								}
-								if ((str_search(cgi_info.input_buffer, hs_conlen) != NULL) || (session->keep_alive == false)) {
+								if ((strcasestr(cgi_info.input_buffer, hs_conlen) != NULL) || (session->keep_alive == false)) {
 									send_in_chunks = false;
 								} else if (send_buffer(session, chunked, 28) == -1) {
 									retval = rs_DISCONNECT;
@@ -629,6 +723,9 @@ int run_cgi_program(t_session *session) {
 												break;
 											}
 										}
+									} else {
+										//retval = rs_QUIT;
+										//break;
 									}
 								} else {
 									if (send_buffer(session, cgi_info.input_buffer, cgi_info.input_len) == -1) {
@@ -645,7 +742,7 @@ int run_cgi_program(t_session *session) {
 							}
 						}
 						break;
-					case rc_END_OF_DATA:
+					case cd_END_OF_DATA:
 						if (in_body) {
 							retval = rs_QUIT;
 							if (send_in_chunks) {
@@ -654,27 +751,28 @@ int run_cgi_program(t_session *session) {
 								}
 							}
 						} else {
+							log_error(session, "target,run_script,cd_END_OF_DATA");
 							retval = 500;
 						}
 						break;
-					case rc_NOTHING:
+					case cd_NOTHING:
 						break;
-					case rc_READ_ERROR:
+					case cd_READ_ERROR:
 						retval = rs_DISCONNECT;
 						break;
 				} // switch
 
 				switch (cgi_info.error_result) {
-					case rc_READ_DATA:
+					case cd_READ_DATA:
 						*(cgi_info.error_buffer + cgi_info.error_len) = '\0';
-						log_CGI_error(session->host->error_logfile, session->file_on_disk, cgi_info.error_buffer);
+						log_cgi_error(session->host->error_logfile, session->file_on_disk, cgi_info.error_buffer);
 						cgi_info.error_len = 0;
 						break;
-					case rc_END_OF_DATA:
+					case cd_END_OF_DATA:
 						break;
-					case rc_NOTHING:
+					case cd_NOTHING:
 						break;
-					case rc_READ_ERROR:
+					case cd_READ_ERROR:
 						break;
 				} // switch
 		} // switch
@@ -688,14 +786,17 @@ int run_cgi_program(t_session *session) {
 			retval = 200;
 	}
 
-	kill(CGI_pid, 9);
-	close(cgi_info.to_cgi);
-	close(cgi_info.from_cgi);
-	close(cgi_info.cgi_error);
+	if (use_fast_cgi) {
+		close(cgi_info.from_cgi);
+	} else {
+		kill(cgi_pid, 9);
 
-	if (cgi_info.input_buffer != NULL) {
-		free(cgi_info.input_buffer);
+		close(cgi_info.to_cgi);
+		close(cgi_info.from_cgi);
+		close(cgi_info.cgi_error);
 	}
+
+	free(cgi_info.input_buffer);
 	free(cgi_info.error_buffer);
 
 	return retval;
@@ -703,21 +804,38 @@ int run_cgi_program(t_session *session) {
 
 /* Converts a filesize to a string.
  */
-static void filesize2str(char *buffer, off_t fsize) {
+static int filesize2str(char *buffer, off_t fsize) {
+	//unsigned long long fsize;
+	int result = 0;
+
+	//fsize = (unsigned long)size;
+
+	buffer[FILESIZE_BUFFER_SIZE - 1] = '\0';
 	if (fsize < 1024) {
-		sprintf(buffer, "%d byte", (int)fsize);
+		result = snprintf(buffer, FILESIZE_BUFFER_SIZE - 1, "%llu byte", (long long)fsize);
 	} else if (fsize < 1048576) {
-		sprintf(buffer, "%0.1f kB", (double)fsize / 1024);
+		result = snprintf(buffer, FILESIZE_BUFFER_SIZE - 1, "%0.1f kB", ((double)(fsize >> 6)) / 16);
+	} else if (fsize < 1073741824) {
+		result = snprintf(buffer, FILESIZE_BUFFER_SIZE - 1, "%0.1f MB", ((double)(fsize >> 16)) / 16);
 	} else {
-		sprintf(buffer, "%0.1f MB", (double)fsize / 1048576);
+		result = snprintf(buffer, FILESIZE_BUFFER_SIZE - 1, "%0.1f GB", ((double)(fsize >> 26)) / 16);
 	}
+
+	return (result < 0) ? 0 : result;
+/*
+	} else {
+		sprintf(buffer, "> 4 GB");
+		return 6;
+	}
+*/
 }
 
 /* Read the content of a directory and send it in a formatted list to the client.
  */
 int show_index(t_session *session) {
-	int total_files = 0, total_fsize = 0, retval = -1;
-	char size[32], color = '0', timestr[33], *slash;
+	int total_files = 0, retval = -1, len;
+	off_t total_fsize = 0;
+	char size[FILESIZE_BUFFER_SIZE], color = '0', timestr[33], *slash;
 	struct tm s;
 	t_filelist *filelist = NULL;
 	bool root_dir;
@@ -762,6 +880,12 @@ int show_index(t_session *session) {
 		}
 	}
 
+    /* HTTP/1.0 has no knowledge about chunked Transfer-Encoding.
+	 */
+	if (*(session->http_version + 7) == '0') {
+		session->keep_alive = false;
+	}
+
 	if (send_header(session, 200) == -1) {
 		return -1;
 	}
@@ -876,8 +1000,8 @@ int show_index(t_session *session) {
 				break;
 			}
 			if (filelist->is_dir == false) {
-				filesize2str(size, filelist->size);
-				if (send_chunk(session, size, strlen(size)) == -1) {
+				len = filesize2str(size, filelist->size);
+				if (send_chunk(session, size, len) == -1) {
 					break;
 				}
 			}
@@ -897,19 +1021,21 @@ int show_index(t_session *session) {
 		if (send_chunk(session,	"<tr class=\"bottom\"><td></td><td class=\"totalfiles\">", 51) == -1) {
 			break;
 		}
-		sprintf(size, "%d", total_files);
+		snprintf(size, FILESIZE_BUFFER_SIZE - 1, "%d", total_files);
 		if (send_chunk(session, size, strlen(size)) == -1) {
 			break;
 		} else if (send_chunk(session, " files</td><td></td><td class=\"totalsize\">", 42) == -1) {
 			break;
 		}
-		filesize2str(size, total_fsize);
-		if (send_chunk(session, size, strlen(size)) == -1) {
+		len = filesize2str(size, total_fsize);
+		if (send_chunk(session, size, len) == -1) {
 			break;
-		} else if (send_chunk(session, "</td><td></td></tr>\n</table></body></html>\n", 42) == -1) {
-			break;
-		} else if (send_chunk(session, NULL, 0) == -1) {
+		} else if (send_chunk(session, "</td><td></td></tr>\n</table></body></html>\n", 43) == -1) {
 			break;
+		} else  if (session->keep_alive) {
+			if (send_chunk(session, NULL, 0) == -1) {
+				break;
+			}
 		}
 
 		retval = 200;
@@ -982,7 +1108,8 @@ int handle_options_request(t_session *session) {
 }
 
 int handle_trace_request(t_session *session) {
-	int result = -1, code, body_size, len;
+	int result = -1, code, body_size;
+	size_t len;
 	char buffer[MAX_TRACE_HEADER + 2];
 	t_headerfield *header;
 
@@ -1003,7 +1130,7 @@ int handle_trace_request(t_session *session) {
 
 	do {
 		// Header
-		if (sprintf(buffer, "%d\r\nContent-Type: message/http\r\n\r\n", body_size) < 0) {
+		if (snprintf(buffer, MAX_TRACE_HEADER, "%d\r\nContent-Type: message/http\r\n\r\n", body_size) < 0) {
 			break;
 		} else if (send_header(session, 200) == -1) {
 			break;
@@ -1019,13 +1146,15 @@ int handle_trace_request(t_session *session) {
 		} else if (code >= MAX_TRACE_HEADER) {
 			break;
 		} else if (session->vars != NULL) {
-			if ((code = snprintf(buffer + strlen(buffer), MAX_TRACE_HEADER, "?%s", session->vars)) < 0) {
+			len = strlen(buffer);
+			if ((code = snprintf(buffer + len, MAX_TRACE_HEADER - len, "?%s", session->vars)) < 0) {
 				break;
 			} else if (code >= MAX_TRACE_HEADER) {
 				break;
 			}
 		}
-		if ((code = snprintf(buffer + strlen(buffer), MAX_TRACE_HEADER, " %s\r\n", session->http_version)) < 0) {
+		len = strlen(buffer);
+		if ((code = snprintf(buffer + len, MAX_TRACE_HEADER - len, " %s\r\n", session->http_version)) < 0) {
 			break;
 		} else if (send_buffer(session, buffer, strlen(buffer)) == -1) {
 			break;
